@@ -17,9 +17,9 @@
  */
 #include "bstrackitem.h"
 
-#include "bststatusbar.h"
 #include "bstmenus.h"
 
+#include <string.h>
 
 /* --- prototypes --- */
 static void	  bst_rack_item_class_init	(BstRackItemClass	*klass);
@@ -32,7 +32,7 @@ static void	  bst_rack_item_button_press	(BstRackItem		*item,
 static GtkWidget* create_controller_menu	(void);
 static void	  sensitize_controller_menu	(GtkWidget		*menu,
 						 GParamSpec		*pspec);
-static BstControllerInfo* controller_info_from_menu_id (guint id);
+static const gchar* controller_info_from_menu_id (guint id);
 
 
 /* --- static variables --- */
@@ -101,8 +101,7 @@ bst_rack_item_init (BstRackItem *item)
   item->block_updates = 0;
   item->pocket = 0;
   item->entry = 0;
-  item->cinfo = NULL;
-  item->cwidget = NULL;
+  item->bparam = NULL;
 }
 
 static void
@@ -110,10 +109,11 @@ bst_rack_item_destroy (GtkObject *object)
 {
   BstRackItem *item = BST_RACK_ITEM (object);
 
-  if (item->cwidget)
-    gtk_widget_destroy (item->cwidget);
-  item->cwidget = NULL;
-  item->cinfo = NULL;
+  if (item->bparam)
+    {
+      bst_param_destroy (item->bparam);
+      item->bparam = NULL;
+    }
 
   bst_rack_item_set_property (item, 0, 0);
 
@@ -162,7 +162,7 @@ bst_rack_item_button_press (BstRackItem    *item,
 {
   if (event->button == 3)
     {
-      gboolean can_clone = item->pocket && item->cinfo && item->proxy && item->pspec;
+      gboolean can_clone = item->pocket && item->bparam && item->proxy && item->pspec;
       guint id;
 
       if (!item->choice)
@@ -187,47 +187,46 @@ bst_rack_item_button_press (BstRackItem    *item,
       id = bst_choice_modal (item->choice, event->button, event->time);
       switch (id)
 	{
-	  BstControllerInfo *cinfo;
 	  gint t;
 	case 0:
 	  /* menu aborted */
 	  break;
 	default: /* 1.. are from controller submenu */
-	  cinfo = controller_info_from_menu_id (id);
-	  bsw_data_pocket_set_string (item->pocket, item->entry, "property-controller", cinfo->name);
+	  bse_data_pocket_set_string (item->pocket, item->entry, "property-controller",
+				      controller_info_from_menu_id (id));
 	  break;
 	case 1001:
-	  t = bsw_data_pocket_get_int (item->pocket, item->entry, "property-hspan");
+	  t = bse_data_pocket_get_int (item->pocket, item->entry, "property-hspan");
 	  t += 1;
 	  t = MAX (t, 1);
-	  bsw_data_pocket_set_int (item->pocket, item->entry, "property-hspan", t);
+	  bse_data_pocket_set_int (item->pocket, item->entry, "property-hspan", t);
 	  break;
 	case 1002:
-	  t = bsw_data_pocket_get_int (item->pocket, item->entry, "property-vspan");
+	  t = bse_data_pocket_get_int (item->pocket, item->entry, "property-vspan");
 	  t += 1;
 	  t = MAX (t, 1);
-	  bsw_data_pocket_set_int (item->pocket, item->entry, "property-vspan", t);
+	  bse_data_pocket_set_int (item->pocket, item->entry, "property-vspan", t);
 	  break;
 	case 1003:
-	  t = bsw_data_pocket_get_int (item->pocket, item->entry, "property-hspan");
+	  t = bse_data_pocket_get_int (item->pocket, item->entry, "property-hspan");
 	  t -= 1;
 	  t = MAX (t, 1);
-	  bsw_data_pocket_set_int (item->pocket, item->entry, "property-hspan", t);
+	  bse_data_pocket_set_int (item->pocket, item->entry, "property-hspan", t);
 	  break;
 	case 1004:
-	  t = bsw_data_pocket_get_int (item->pocket, item->entry, "property-vspan");
+	  t = bse_data_pocket_get_int (item->pocket, item->entry, "property-vspan");
 	  t -= 1;
 	  t = MAX (t, 1);
-	  bsw_data_pocket_set_int (item->pocket, item->entry, "property-vspan", t);
+	  bse_data_pocket_set_int (item->pocket, item->entry, "property-vspan", t);
 	  break;
 	case 1005:
-	  id = bsw_data_pocket_create_entry (item->pocket);
-	  bsw_data_pocket_set_string (item->pocket, id, "property-controller", item->cinfo->name);
-	  bsw_data_pocket_set_object (item->pocket, id, "property-object", item->proxy);
-	  bsw_data_pocket_set_string (item->pocket, id, "property-name", item->pspec->name);
+	  id = bse_data_pocket_create_entry (item->pocket);
+	  bse_data_pocket_set_string (item->pocket, id, "property-controller", bst_param_get_view_name (item->bparam));
+	  bse_data_pocket_set_object (item->pocket, id, "property-object", item->proxy);
+	  bse_data_pocket_set_string (item->pocket, id, "property-name", item->pspec->name);
 	  break;
 	case 1006:
-	  bsw_data_pocket_delete_entry (item->pocket, item->entry);
+	  bse_data_pocket_delete_entry (item->pocket, item->entry);
 	  break;
 	}
     }
@@ -239,31 +238,25 @@ pocket_entry_changed (BstRackItem *item,
 {
   if (item->entry == entry && !item->block_updates)
     {
-      BstControllerInfo *cinfo = NULL;
       GParamSpec *pspec = NULL;
-      BswProxy proxy = 0;
-      gchar *controller;
+      SfiProxy proxy = 0;
+      const gchar *controller, *name;
 
-      controller = bsw_data_pocket_get_string (item->pocket, item->entry, "property-controller");
-      cinfo = bst_controller_lookup (controller, NULL);
-      if (cinfo)
-	{
-	  gchar *name = bsw_data_pocket_get_string (item->pocket, item->entry, "property-name");
+      controller = bse_data_pocket_get_string (item->pocket, item->entry, "property-controller");
+      name = bse_data_pocket_get_string (item->pocket, item->entry, "property-name");
+      proxy = bse_data_pocket_get_object (item->pocket, item->entry, "property-object");
+      if (proxy && name)
+	pspec = bse_proxy_get_pspec (proxy, name);
+      bst_rack_item_set_proxy (item, pspec ? proxy : 0, pspec, controller);
 
-	  proxy = bsw_data_pocket_get_object (item->pocket, item->entry, "property-object");
-	  if (proxy && name)
-	    pspec = bsw_proxy_get_pspec (proxy, name);
-	}
-      bst_rack_item_set_proxy (item, pspec ? proxy : 0, pspec, cinfo);
-
-      if (item->cinfo)
+      if (item->bparam)
 	{
 	  BstRackChildInfo info;
 
-	  info.col = bsw_data_pocket_get_int (item->pocket, item->entry, "property-x");
-	  info.row = bsw_data_pocket_get_int (item->pocket, item->entry, "property-y");
-	  info.hspan = bsw_data_pocket_get_int (item->pocket, item->entry, "property-hspan");
-	  info.vspan = bsw_data_pocket_get_int (item->pocket, item->entry, "property-vspan");
+	  info.col = bse_data_pocket_get_int (item->pocket, item->entry, "property-x");
+	  info.row = bse_data_pocket_get_int (item->pocket, item->entry, "property-y");
+	  info.hspan = bse_data_pocket_get_int (item->pocket, item->entry, "property-hspan");
+	  info.vspan = bse_data_pocket_get_int (item->pocket, item->entry, "property-vspan");
 	  if ((info.col != item->rack_child_info.col ||
 	       info.row != item->rack_child_info.row ||
 	       info.hspan != item->rack_child_info.hspan ||
@@ -282,19 +275,19 @@ rack_item_remove (BstRackItem *item)
 
 void
 bst_rack_item_set_property (BstRackItem *item,
-			    BswProxy     pocket,
+			    SfiProxy     pocket,
 			    guint        entry_id)
 {
   g_return_if_fail (BST_IS_RACK_ITEM (item));
   if (pocket)
     {
-      g_return_if_fail (BSW_IS_DATA_POCKET (pocket));
+      g_return_if_fail (BSE_IS_DATA_POCKET (pocket));
       g_return_if_fail (entry_id > 0);
     }
 
   if (item->pocket)
     {
-      bsw_proxy_disconnect (item->pocket,
+      bse_proxy_disconnect (item->pocket,
 			    "any_signal", rack_item_remove, item,
 			    "any_signal", pocket_entry_changed, item,
 			    NULL);
@@ -304,8 +297,8 @@ bst_rack_item_set_property (BstRackItem *item,
   item->entry = entry_id;
   if (item->pocket)
     {
-      bsw_proxy_connect (item->pocket,
-			 "swapped_signal::set_parent", rack_item_remove, item,
+      bse_proxy_connect (item->pocket,
+			 "swapped_signal::release", rack_item_remove, item,
 			 "swapped_signal::entry-changed", pocket_entry_changed, item,
 			 NULL);
     }
@@ -314,57 +307,57 @@ bst_rack_item_set_property (BstRackItem *item,
 }
 
 void
-bst_rack_item_set_proxy (BstRackItem       *item,
-			 BswProxy	    proxy,
-			 GParamSpec        *pspec,
-			 BstControllerInfo *cinfo)
+bst_rack_item_set_proxy (BstRackItem	*item,
+			 SfiProxy	 proxy,
+			 GParamSpec	*pspec,
+			 const gchar	*view_name)
 {
   g_return_if_fail (BST_IS_RACK_ITEM (item));
   if (pspec)
     g_return_if_fail (G_IS_PARAM_SPEC (pspec));
 
+  view_name = pspec ? bst_param_lookup_view (pspec, TRUE, view_name, bst_param_binding_proxy ()) : NULL;
+
   /* have to optimize this for non-changes */
-  if (item->proxy != proxy || item->pspec != pspec || item->cinfo != cinfo)
+  if (item->proxy != proxy ||
+      item->pspec != pspec ||
+      (!view_name && item->bparam) ||
+      (view_name && !item->bparam) ||
+      (view_name && item->bparam &&
+       strcmp (view_name, bst_param_get_view_name (item->bparam)) != 0))
     {
-      if (pspec && !bst_controller_check (cinfo, pspec))
-	cinfo = NULL;
-
-      if (item->cwidget)
-	gtk_widget_destroy (item->cwidget);
-      item->cinfo = cinfo;
-      
-      if (item->proxy)
-	bsw_proxy_disconnect (item->proxy,
-			      "any_signal", bst_rack_item_model_changed, item,
-			      NULL);
-      item->proxy = pspec && cinfo ? proxy : 0;
-      item->pspec = item->proxy ? pspec : NULL;
-      if (item->proxy)
+      if (item->bparam)
 	{
-	  gchar *name = g_strdup_printf ("swapped_signal::notify::%s", item->pspec->name);
-
-	  bsw_proxy_connect (item->proxy,
-			     name, bst_rack_item_model_changed, item,
-			     NULL);
-	  g_free (name);
-	  item->cwidget = bst_controller_create (cinfo, item->pspec,
-						 (GCallback) bst_rack_item_controler_changed,
-						 item);
-	  g_object_connect (item->cwidget,
-			    "swapped_signal::destroy", g_nullify_pointer, &item->cwidget,
-			    NULL);
-	  gtk_container_add (GTK_CONTAINER (item), item->cwidget);
-	  gtk_widget_show (item->cwidget);
+	  bst_param_destroy (item->bparam);
+	  item->bparam = NULL;
 	}
 
-      if (item->cinfo)
+      if (item->pspec)
+	g_param_spec_unref (item->pspec);
+      if (proxy && pspec && view_name)
 	{
-	  gchar *name = bsw_data_pocket_get_string (item->pocket, item->entry, "property-controller");
+	  item->proxy = proxy;
+	  item->pspec = pspec;
+	}
+      else
+	{
+	  item->proxy = 0;
+	  item->pspec = NULL;
+	}
+      if (item->pspec)
+	g_param_spec_ref (item->pspec);
 
-	  if (!name || strcmp (name, cinfo->name) != 0)
-	    bsw_data_pocket_set_string (item->pocket, item->entry, "property-controller", cinfo->name);
+      if (item->proxy)
+	{
+	  const gchar *name = bse_data_pocket_get_string (item->pocket, item->entry, "property-controller");
+	  item->bparam = bst_param_proxy_create (item->pspec, TRUE, view_name, item->proxy);
+	  // notify via: (GCallback) bst_rack_item_controler_changed, item
+	  gtk_container_add (GTK_CONTAINER (item), bst_param_rack_widget (item->bparam));
+	  if (!name || strcmp (name, view_name) != 0)
+	    bse_data_pocket_set_string (item->pocket, item->entry, "property-controller", view_name);
+	  bst_param_update (item->bparam);
+          g_return_if_fail (item->bparam->binding != NULL);
 	}
-      bst_rack_item_model_changed (item);
     }
 }
 
@@ -373,75 +366,29 @@ bst_rack_item_gui_changed (BstRackItem *item)
 {
   g_return_if_fail (BST_IS_RACK_ITEM (item));
 
-  if (item->pocket && item->entry && item->cinfo && !item->block_updates)
+  if (item->pocket && item->entry && item->bparam && !item->block_updates)
     {
       item->block_updates++;
-      bsw_data_pocket_set_int (item->pocket, item->entry, "property-x", item->rack_child_info.col);
-      bsw_data_pocket_set_int (item->pocket, item->entry, "property-y", item->rack_child_info.row);
-      bsw_data_pocket_set_int (item->pocket, item->entry, "property-hspan", item->rack_child_info.hspan);
-      bsw_data_pocket_set_int (item->pocket, item->entry, "property-vspan", item->rack_child_info.vspan);
+      bse_data_pocket_set_int (item->pocket, item->entry, "property-x", item->rack_child_info.col);
+      bse_data_pocket_set_int (item->pocket, item->entry, "property-y", item->rack_child_info.row);
+      bse_data_pocket_set_int (item->pocket, item->entry, "property-hspan", item->rack_child_info.hspan);
+      bse_data_pocket_set_int (item->pocket, item->entry, "property-vspan", item->rack_child_info.vspan);
       item->block_updates--;
     }
 }
 
-void
-bst_rack_item_controler_changed (BstRackItem *item)
-{
-  g_return_if_fail (BST_IS_RACK_ITEM (item));
-
-  if (item->cwidget && item->proxy && item->pspec && !item->block_updates)
-    {
-      item->block_updates++;
-      if (item->cinfo->fetch)
-	{
-	  GValue value = { 0, };
-	  GType type = item->cinfo->value_type ? item->cinfo->value_type : G_PARAM_SPEC_VALUE_TYPE (item->pspec);
-	  
-	  g_value_init (&value, type);
-	  bst_controller_fetch (item->cwidget, &value);
-	  g_object_set_property (bse_object_from_id (item->proxy), item->pspec->name, &value);
-	  g_value_unset (&value);
-	}
-      item->block_updates--;
-    }
-}
-
-void
-bst_rack_item_model_changed (BstRackItem *item)
-{
-  g_return_if_fail (BST_IS_RACK_ITEM (item));
-
-  if (item->cwidget && item->proxy && item->pspec)
-    {
-      GValue value = { 0, };
-      GType type = item->cinfo->value_type ? item->cinfo->value_type : G_PARAM_SPEC_VALUE_TYPE (item->pspec);
-
-      g_value_init (&value, type);
-      g_object_get_property (bse_object_from_id (item->proxy), item->pspec->name, &value);
-      item->block_updates++;
-      bst_controller_update (item->cwidget, &value);
-      item->block_updates--;
-      g_value_unset (&value);
-    }
-}
-
 GtkWidget*
 create_controller_menu (void)
 {
+  const gchar **names = bst_param_list_names (TRUE, NULL);
   GtkWidget *menu;
-  GSList *slist = bst_controller_list ();
   guint i;
 
   menu = g_object_new (GTK_TYPE_MENU,
 		       "visible", TRUE,
 		       NULL);
-  for (i = 1; slist; slist = slist->next, i++)
-    {
-      BstControllerInfo *cinfo = slist->data;
-
-      bst_choice_menu_add_choice_and_free (menu, BST_CHOICE (i, cinfo->name, NONE));
-    }
-
+  for (i = 0; names[i]; i++)
+    bst_choice_menu_add_choice_and_free (menu, BST_CHOICE (1 + i, names[i], NONE));
   return menu;
 }
 
@@ -449,17 +396,20 @@ static void
 sensitize_controller_menu (GtkWidget  *menu,
 			   GParamSpec *pspec)
 {
-  GSList *slist;
-  guint id = 1;
+  const gchar **names = bst_param_list_names (TRUE, NULL);
+  guint i;
 
-  for (slist = bst_controller_list (); slist; slist = slist->next)
-    bst_choice_menu_set_item_sensitive (menu, id++, bst_controller_check (slist->data, pspec));
+  for (i = 0; names[i]; i++)
+    bst_choice_menu_set_item_sensitive (menu, 1 + i,
+					bst_param_rate_check (pspec, TRUE, names[i],
+							      bst_param_binding_proxy ()) > 0);
 }
 
-static BstControllerInfo*
+static const gchar*
 controller_info_from_menu_id (guint id)
 {
-  g_return_val_if_fail (id > 0, NULL);
+  guint n;
+  const gchar **names = bst_param_list_names (TRUE, &n);
 
-  return g_slist_nth_data (bst_controller_list (), id - 1);
+  return id > 0 && id <= n ? names[id - 1] : NULL;
 }
