@@ -16,56 +16,70 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include "bsttexttools.h"
-#include "bstdialog.h"
+#include <gxk/gxk.h>
 
+#include "../PKG_paths.h"
+
+#include <string.h>
 
 #define PRGNAME "tsmview"
 
-gboolean            bst_dvl_hints = FALSE;
-
+/* --- functions --- */
 int
 main (int   argc,
       char *argv[])
 {
   GtkWidget *sctext, *dialog;
-  gchar *str, *title;
-  guint i;
-
-  if (argc < 2)
-    {
-      g_printerr (PRGNAME ": missing filename arguments\n");
-      return 1;
-    }
+  gchar *str, *title = NULL;
+  guint i, flags = 0;
 
   /* initialize modules
    */
   g_type_init ();
-  _bst_utils_init ();
-  bst_init_gentypes ();
   gtk_init (&argc, &argv);
+  gxk_init ();
 
-  title = g_strdup (argv[1]);
-  bst_text_add_tsm_path (".");
-  sctext = bst_scroll_text_from_file (BST_TEXT_VIEW_PARSE_TSM | BST_TEXT_VIEW_SHEET_BG, argv[1]);
-  for (i = 2; i < argc; i++)
-    {
-      bst_scroll_text_append_file_tsm (sctext, argv[i]);
-      str = title;
-      title = g_strconcat (title, " ", argv[i], NULL);
-      g_free (str);
-    }
-
+  for (i = 1; i < argc; i++)
+    if (!flags && strcmp (argv[i], "--edit") == 0)
+      {
+	flags = GXK_SCROLL_TEXT_EDITABLE;
+	argv[i] = NULL;
+	if (title)
+	  break;
+      }
+    else if (!title)
+      {
+	title = g_strdup (argv[i]);
+	argv[i] = NULL;
+	if (flags)
+	  break;
+      }
+  if (!title)
+    title = g_strdup (".");
+  gxk_text_add_tsm_path (".");
+  gxk_text_add_tsm_path (BST_PATH_DOCS);
+  gxk_text_add_tsm_path (BST_PATH_IMAGES);
+  sctext = gxk_scroll_text_create (GXK_SCROLL_TEXT_NAVIGATABLE | flags, NULL);
+  gxk_scroll_text_enter (sctext, title);
+  for (i = 1; i < argc; i++)
+    if (argv[i])
+      {
+	gxk_scroll_text_enter (sctext, argv[i]);
+	str = title;
+	title = g_strconcat (title, " ", argv[i], NULL);
+	g_free (str);
+      }
+  
   str = title;
-  title = g_strconcat (title, " - tsmview", NULL);
+  title = g_strdup ("tsmview");	// g_strconcat (title, " - tsmview", NULL);
   g_free (str);
 
-  dialog = bst_dialog_new (NULL, NULL, BST_DIALOG_DELETE_BUTTON, title, NULL);
+  dialog = gxk_dialog_new (NULL, NULL, GXK_DIALOG_DELETE_BUTTON, title, NULL);
   g_free (title);
 
   g_object_connect (dialog, "signal::destroy", gtk_main_quit, NULL, NULL);
 
-  bst_dialog_set_child (BST_DIALOG (dialog), sctext);
+  gxk_dialog_set_child (GXK_DIALOG (dialog), sctext);
 
   g_object_set (dialog,
 		"default_width", 560,
