@@ -14,20 +14,21 @@
 #include <bse/bsejanitor.h>
 #include <bse/bsescripthelper.h>
 #include <bse/bsegconfig.h>
+#include <bse/bseladspa.h>
 #include "PKG_config.h"
 #include <string.h>
 
 
-#line 34 "bseserver.proc"
+#line 35 "bseserver.proc"
 
 
 /* --- use-new-project --- */
 static GType type_id_use_new_project = 0;
 static void
 use_new_project_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 36 "bseserver.proc"
+#line 37 "bseserver.proc"
  {
-#line 38 "bseserver.proc"
+#line 39 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
   *(in_pspecs++) = sfi_pspec_string ("name", "Project Name", NULL,
@@ -35,22 +36,22 @@ use_new_project_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSp
   *(out_pspecs++) = bse_param_spec_object ("project", "Project", NULL,
 				 BSE_TYPE_PROJECT, SFI_PARAM_DEFAULT);
 }
-#line 37 "bseserver.proc"
+#line 38 "bseserver.proc"
   proc->help = "Create a new project, owned by the caller (name is modified to be unique if necessary)";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 45 "bseserver.proc"
+#line 46 "bseserver.proc"
  }
 static BseErrorType
-#line 45 "bseserver.proc"
+#line 46 "bseserver.proc"
 use_new_project_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 48 "bseserver.proc"
-{
 #line 49 "bseserver.proc"
+{
+#line 50 "bseserver.proc"
   
   BseServer *server     = (BseServer*) bse_value_get_object (in_values++);
   gchar *name           = sfi_value_get_string (in_values++);
@@ -58,12 +59,12 @@ use_new_project_exec (BseProcedureClass *proc,
   guint num = 1;
   BseProject *project;
   
-#line 56 "bseserver.proc"
+#line 57 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server) || !name)
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 60 "bseserver.proc"
+#line 61 "bseserver.proc"
   
   uname = g_strdup (name);
   while (bse_server_find_project (server, uname))
@@ -72,14 +73,14 @@ use_new_project_exec (BseProcedureClass *proc,
       uname = g_strdup_printf ("%s-%u", name, num++);
     }
   
-#line 68 "bseserver.proc"
+#line 69 "bseserver.proc"
   
   project = bse_server_create_project (server, uname);
   g_free (uname);
   bse_item_use (BSE_ITEM (project));
   g_object_unref (project);
   
-#line 74 "bseserver.proc"
+#line 75 "bseserver.proc"
   
   bse_value_set_object (out_values++, G_OBJECT (project));
   
@@ -90,36 +91,36 @@ use_new_project_exec (BseProcedureClass *proc,
 static GType type_id_get_midi_notifier = 0;
 static void
 get_midi_notifier_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 80 "bseserver.proc"
+#line 81 "bseserver.proc"
  {
-#line 82 "bseserver.proc"
+#line 83 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
   *(out_pspecs++) = bse_param_spec_object ("midi_notifier", "Midi Notifier", NULL,
 				 BSE_TYPE_MIDI_NOTIFIER, SFI_PARAM_DEFAULT);
 }
-#line 81 "bseserver.proc"
+#line 82 "bseserver.proc"
   proc->help = "Retrieve the global midi notifier object.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 87 "bseserver.proc"
+#line 88 "bseserver.proc"
  }
 static BseErrorType
-#line 87 "bseserver.proc"
+#line 88 "bseserver.proc"
 get_midi_notifier_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 90 "bseserver.proc"
-{
 #line 91 "bseserver.proc"
+{
+#line 92 "bseserver.proc"
   
   BseServer *self = bse_value_get_object (in_values++);
   BseMidiReceiver *midi_receiver;
   BseMidiNotifier *notifier;
   
-#line 96 "bseserver.proc"
+#line 97 "bseserver.proc"
   
   if (!BSE_IS_SERVER (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -133,7 +134,7 @@ get_midi_notifier_exec (BseProcedureClass *proc,
       g_object_unref (notifier);
     }
   
-#line 109 "bseserver.proc"
+#line 110 "bseserver.proc"
   
   bse_value_set_object (out_values++, G_OBJECT (notifier));
   
@@ -144,9 +145,9 @@ get_midi_notifier_exec (BseProcedureClass *proc,
 static GType type_id_can_load = 0;
 static void
 can_load_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 115 "bseserver.proc"
+#line 116 "bseserver.proc"
  {
-#line 117 "bseserver.proc"
+#line 118 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
   *(in_pspecs++)    = sfi_pspec_string ("file_name", "File Name", "The file to find a loader for",
@@ -154,33 +155,33 @@ can_load_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **ou
   *(out_pspecs++)   = sfi_pspec_bool ("can-load", NULL, NULL,
 			  FALSE, SFI_PARAM_DEFAULT);
 }
-#line 116 "bseserver.proc"
+#line 117 "bseserver.proc"
   proc->help = "Check whether a loader can be found for a wave file";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 124 "bseserver.proc"
+#line 125 "bseserver.proc"
  }
 static BseErrorType
-#line 124 "bseserver.proc"
+#line 125 "bseserver.proc"
 can_load_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 127 "bseserver.proc"
-{
 #line 128 "bseserver.proc"
+{
+#line 129 "bseserver.proc"
   
   BseServer *server	= bse_value_get_object (in_values++);
   gchar *file_name	= sfi_value_get_string (in_values++);
   GslWaveFileInfo *finfo;
   
-#line 133 "bseserver.proc"
+#line 134 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server) || !file_name)
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 137 "bseserver.proc"
+#line 138 "bseserver.proc"
   
   finfo = gsl_wave_file_info_load (file_name, NULL);
   sfi_value_set_bool (out_values++, finfo != NULL);
@@ -194,9 +195,9 @@ can_load_exec (BseProcedureClass *proc,
 static GType type_id_describe_note = 0;
 static void
 describe_note_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 146 "bseserver.proc"
+#line 147 "bseserver.proc"
  {
-#line 148 "bseserver.proc"
+#line 149 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
   *(in_pspecs++)    = bse_pspec_note_simple ("note", "Note", NULL, SFI_PARAM_DEFAULT);
@@ -204,33 +205,33 @@ describe_note_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec
   *(out_pspecs++)   = bse_param_spec_boxed ("note-description", "Note Description", NULL,
 			 BSE_TYPE_NOTE_DESCRIPTION, SFI_PARAM_DEFAULT);
 }
-#line 147 "bseserver.proc"
+#line 148 "bseserver.proc"
   proc->help = "Describe a note, providing information about its octave, semitone, frequency, etc.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 155 "bseserver.proc"
+#line 156 "bseserver.proc"
  }
 static BseErrorType
-#line 155 "bseserver.proc"
+#line 156 "bseserver.proc"
 describe_note_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 158 "bseserver.proc"
-{
 #line 159 "bseserver.proc"
+{
+#line 160 "bseserver.proc"
   
   BseServer *server	= bse_value_get_object (in_values++);
   gint       note	= sfi_value_get_int (in_values++);
   gint       fine_tune	= sfi_value_get_int (in_values++);
   
-#line 164 "bseserver.proc"
+#line 165 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 168 "bseserver.proc"
+#line 169 "bseserver.proc"
   
   bse_value_take_boxed (out_values++, bse_note_description (note, fine_tune));
   
@@ -241,9 +242,9 @@ describe_note_exec (BseProcedureClass *proc,
 static GType type_id_note_from_freq = 0;
 static void
 note_from_freq_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 174 "bseserver.proc"
+#line 175 "bseserver.proc"
  {
-#line 176 "bseserver.proc"
+#line 177 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
   *(in_pspecs++)    = sfi_pspec_real ("freq", "Frequency", NULL,
@@ -253,33 +254,33 @@ note_from_freq_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpe
   *(out_pspecs++)   = bse_param_spec_boxed ("note-description", "Note Description", NULL,
 			 BSE_TYPE_NOTE_DESCRIPTION, SFI_PARAM_DEFAULT);
 }
-#line 175 "bseserver.proc"
+#line 176 "bseserver.proc"
   proc->help = "Describe a note, given its frequency.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 185 "bseserver.proc"
+#line 186 "bseserver.proc"
  }
 static BseErrorType
-#line 185 "bseserver.proc"
+#line 186 "bseserver.proc"
 note_from_freq_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 188 "bseserver.proc"
-{
 #line 189 "bseserver.proc"
+{
+#line 190 "bseserver.proc"
   
   BseServer *server	= bse_value_get_object (in_values++);
   gfloat     freq	= sfi_value_get_real (in_values++);
   gint note;
   
-#line 194 "bseserver.proc"
+#line 195 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 198 "bseserver.proc"
+#line 199 "bseserver.proc"
   
   note = bse_note_from_freq (freq);
   bse_value_take_boxed (out_values++, bse_note_description (note, 0));
@@ -291,9 +292,9 @@ note_from_freq_exec (BseProcedureClass *proc,
 static GType type_id_note_from_string = 0;
 static void
 note_from_string_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 205 "bseserver.proc"
+#line 206 "bseserver.proc"
  {
-#line 207 "bseserver.proc"
+#line 208 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
   *(in_pspecs++)    = sfi_pspec_string ("name", "Name", NULL,
@@ -301,33 +302,33 @@ note_from_string_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamS
   *(out_pspecs++)   = bse_param_spec_boxed ("note-description", "Note Description", NULL,
 			 BSE_TYPE_NOTE_DESCRIPTION, SFI_PARAM_DEFAULT);
 }
-#line 206 "bseserver.proc"
+#line 207 "bseserver.proc"
   proc->help = "Describe a note, given its name and octave offset.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 214 "bseserver.proc"
+#line 215 "bseserver.proc"
  }
 static BseErrorType
-#line 214 "bseserver.proc"
+#line 215 "bseserver.proc"
 note_from_string_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 217 "bseserver.proc"
-{
 #line 218 "bseserver.proc"
+{
+#line 219 "bseserver.proc"
   
   BseServer *server	= bse_value_get_object (in_values++);
   gchar     *name     	= sfi_value_get_string (in_values++);
   gint note;
   
-#line 223 "bseserver.proc"
+#line 224 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server) || !name)
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 227 "bseserver.proc"
+#line 228 "bseserver.proc"
   
   note = bse_note_from_string (name);
   bse_value_take_boxed (out_values++, bse_note_description (note, 0));
@@ -339,9 +340,9 @@ note_from_string_exec (BseProcedureClass *proc,
 static GType type_id_construct_note = 0;
 static void
 construct_note_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 234 "bseserver.proc"
+#line 235 "bseserver.proc"
  {
-#line 236 "bseserver.proc"
+#line 237 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
   *(in_pspecs++)    = sfi_pspec_int ("semitone", NULL, NULL,
@@ -352,22 +353,22 @@ construct_note_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpe
   *(out_pspecs++)   = bse_param_spec_boxed ("note-description", "Note Description", NULL,
 			 BSE_TYPE_NOTE_DESCRIPTION, SFI_PARAM_DEFAULT);
 }
-#line 235 "bseserver.proc"
+#line 236 "bseserver.proc"
   proc->help = "Describe a note, given its semitone, octave and fine tune.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 246 "bseserver.proc"
+#line 247 "bseserver.proc"
  }
 static BseErrorType
-#line 246 "bseserver.proc"
+#line 247 "bseserver.proc"
 construct_note_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 249 "bseserver.proc"
-{
 #line 250 "bseserver.proc"
+{
+#line 251 "bseserver.proc"
   
   BseServer *server	= bse_value_get_object (in_values++);
   guint      semitone	= sfi_value_get_int (in_values++);
@@ -375,12 +376,12 @@ construct_note_exec (BseProcedureClass *proc,
   gint       fine_tune	= sfi_value_get_int (in_values++);
   gint note;
   
-#line 257 "bseserver.proc"
+#line 258 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 261 "bseserver.proc"
+#line 262 "bseserver.proc"
   
   note = BSE_NOTE_GENERIC (octave, semitone);
   bse_value_take_boxed (out_values++, bse_note_description (note, fine_tune));
@@ -392,48 +393,48 @@ construct_note_exec (BseProcedureClass *proc,
 static GType type_id_n_scripts = 0;
 static void
 n_scripts_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 268 "bseserver.proc"
+#line 269 "bseserver.proc"
  {
-#line 270 "bseserver.proc"
+#line 271 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
   *(out_pspecs++)   = sfi_pspec_int ("n_connections", NULL, "Number of Connections",
 			 0, 0, G_MAXINT, 1,
 			 SFI_PARAM_DEFAULT);
 }
-#line 269 "bseserver.proc"
+#line 270 "bseserver.proc"
   proc->help = "Return the number of scripts currently running on this server.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 276 "bseserver.proc"
+#line 277 "bseserver.proc"
  }
 static BseErrorType
-#line 276 "bseserver.proc"
+#line 277 "bseserver.proc"
 n_scripts_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 279 "bseserver.proc"
-{
 #line 280 "bseserver.proc"
+{
+#line 281 "bseserver.proc"
   
   BseServer *server	= bse_value_get_object (in_values++);
   GSList *slist;
   guint n_scripts = 0;
   
-#line 285 "bseserver.proc"
+#line 286 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 289 "bseserver.proc"
+#line 290 "bseserver.proc"
   
   for (slist = server->children; slist; slist = slist->next)
     if (BSE_IS_JANITOR (slist->data))
       n_scripts++;
   
-#line 294 "bseserver.proc"
+#line 295 "bseserver.proc"
   
   sfi_value_set_int (out_values++, n_scripts);
   
@@ -444,39 +445,39 @@ n_scripts_exec (BseProcedureClass *proc,
 static GType type_id_preferences_locked = 0;
 static void
 preferences_locked_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 300 "bseserver.proc"
+#line 301 "bseserver.proc"
  {
-#line 302 "bseserver.proc"
+#line 303 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
   *(out_pspecs++)   = sfi_pspec_bool ("locked", NULL, NULL,
 			  FALSE, SFI_PARAM_DEFAULT);
 }
-#line 301 "bseserver.proc"
+#line 302 "bseserver.proc"
   proc->help = "Returns whether the bse-preferences property is currently locked against modifications or not.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 307 "bseserver.proc"
+#line 308 "bseserver.proc"
  }
 static BseErrorType
-#line 307 "bseserver.proc"
+#line 308 "bseserver.proc"
 preferences_locked_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 310 "bseserver.proc"
-{
 #line 311 "bseserver.proc"
+{
+#line 312 "bseserver.proc"
   
   BseServer *server	= bse_value_get_object (in_values++);
   
-#line 314 "bseserver.proc"
+#line 315 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 318 "bseserver.proc"
+#line 319 "bseserver.proc"
   
   sfi_value_set_int (out_values++, bse_gconfig_locked ());
   
@@ -533,36 +534,36 @@ register_scripts (gpointer data)
 static GType type_id_register_scripts = 0;
 static void
 register_scripts_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 370 "bseserver.proc"
+#line 371 "bseserver.proc"
  {
-#line 372 "bseserver.proc"
+#line 373 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
 }
-#line 371 "bseserver.proc"
+#line 372 "bseserver.proc"
   proc->help = "Register external scripts.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 375 "bseserver.proc"
+#line 376 "bseserver.proc"
  }
 static BseErrorType
-#line 375 "bseserver.proc"
+#line 376 "bseserver.proc"
 register_scripts_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 378 "bseserver.proc"
+#line 379 "bseserver.proc"
 {
   static gboolean registration_done = FALSE;
-#line 380 "bseserver.proc"
+#line 381 "bseserver.proc"
   
   BseServer *server	       = bse_value_get_object (in_values++);
   const gchar *path            = BSE_PATH_SCRIPTS;
   ScriptRegistration *scr_list = NULL;
   GSList *free_list, *list;
 
-#line 386 "bseserver.proc"
+#line 387 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -596,20 +597,31 @@ static gboolean
 register_plugins (gpointer data)
 {
   BseServer *server = data;
-  GSList *slist = g_object_get_data (server, "plugin-registration-queue");
+  GSList *plugin_slist = g_object_get_data (server, "plugin-registration-queue");
+  GSList *ladspa_slist = g_object_get_data (server, "ladspa-registration-queue");
   const gchar *error;
 
-  if (!slist)
+  if (plugin_slist)
+    {
+      g_object_set_data (server, "plugin-registration-queue", plugin_slist->next);
+      error = bse_plugin_check_load (plugin_slist->data);
+      bse_server_registration (server, BSE_REGISTER_PLUGIN, plugin_slist->data, error);
+      g_free (plugin_slist->data);
+      g_slist_free_1 (plugin_slist);
+    }
+  else if (ladspa_slist)
+    {
+      g_object_set_data (server, "ladspa-registration-queue", ladspa_slist->next);
+      error = bse_ladspa_plugin_check_load (ladspa_slist->data);
+      bse_server_registration (server, BSE_REGISTER_PLUGIN, ladspa_slist->data, error);
+      g_free (ladspa_slist->data);
+      g_slist_free_1 (ladspa_slist);
+    }
+  else
     {
       bse_server_registration (server, BSE_REGISTER_DONE, NULL, NULL);
       return FALSE;
     }
-  g_object_set_data (server, "plugin-registration-queue", slist->next);
-
-  error = bse_plugin_check_load (slist->data);
-  bse_server_registration (server, BSE_REGISTER_PLUGIN, slist->data, error);
-  g_free (slist->data);
-  g_slist_free_1 (slist);
   return TRUE;
 }
 
@@ -617,35 +629,35 @@ register_plugins (gpointer data)
 static GType type_id_register_plugins = 0;
 static void
 register_plugins_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 436 "bseserver.proc"
+#line 448 "bseserver.proc"
  {
-#line 438 "bseserver.proc"
+#line 450 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
 }
-#line 437 "bseserver.proc"
+#line 449 "bseserver.proc"
   proc->help = "Register core plugins.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 441 "bseserver.proc"
+#line 453 "bseserver.proc"
  }
 static BseErrorType
-#line 441 "bseserver.proc"
+#line 453 "bseserver.proc"
 register_plugins_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 444 "bseserver.proc"
+#line 456 "bseserver.proc"
 {
   static gboolean registration_done = FALSE;
-#line 446 "bseserver.proc"
+#line 458 "bseserver.proc"
   
   BseServer *server	= bse_value_get_object (in_values++);
-  const gchar *path = BSE_PATH_PLUGINS;
+  gchar *paths;
   GSList *slist;
 
-#line 451 "bseserver.proc"
+#line 463 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -657,8 +669,17 @@ register_plugins_exec (BseProcedureClass *proc,
     }
   registration_done = TRUE;
 
-  slist = bse_plugin_dir_list_files (path);
+  slist = bse_plugin_dir_list_files (BSE_PATH_PLUGINS);
   g_object_set_data (server, "plugin-registration-queue", slist);
+
+  slist = bse_ladspa_plugin_dir_list_files (BSE_PATH_LADSPA);
+  paths = g_getenv ("LADSPA_PATH");
+  if (paths)
+    {
+      GSList *slist2 = bse_ladspa_plugin_dir_list_files (paths);
+      slist = g_slist_concat (slist, slist2);
+    }
+  g_object_set_data (server, "ladspa-registration-queue", slist);
   bse_idle_normal (register_plugins, server);
 
   return BSE_ERROR_NONE;
@@ -673,28 +694,28 @@ register_plugins_exec (BseProcedureClass *proc,
 static GType type_id_save_preferences = 0;
 static void
 save_preferences_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 474 "bseserver.proc"
+#line 495 "bseserver.proc"
  {
-#line 476 "bseserver.proc"
+#line 497 "bseserver.proc"
   *(in_pspecs++) = bse_param_spec_object ("server", "Server", NULL,
 				 BSE_TYPE_SERVER, SFI_PARAM_DEFAULT);
 }
-#line 475 "bseserver.proc"
+#line 496 "bseserver.proc"
   proc->help = "Request the bse-preferences property to be saved into BSE's configuration file.";
-#line 32 "bseserver.proc"
-  proc->authors = "Tim Janik <timj@gtk.org>";
 #line 33 "bseserver.proc"
+  proc->authors = "Tim Janik <timj@gtk.org>";
+#line 34 "bseserver.proc"
   proc->copyright = "Copyright (C) 2001-2002 Tim Janik";
-#line 479 "bseserver.proc"
+#line 500 "bseserver.proc"
  }
 static BseErrorType
-#line 479 "bseserver.proc"
+#line 500 "bseserver.proc"
 save_preferences_exec (BseProcedureClass *proc,
       const GValue      *in_values,
       GValue            *out_values)
-#line 482 "bseserver.proc"
+#line 503 "bseserver.proc"
 {
-#line 483 "bseserver.proc"
+#line 504 "bseserver.proc"
   
   BseServer *server = bse_value_get_object (in_values++);
   SfiWStore *wstore;
@@ -703,14 +724,14 @@ save_preferences_exec (BseProcedureClass *proc,
   SfiRec *rec;
   gint fd;
   
-#line 491 "bseserver.proc"
+#line 512 "bseserver.proc"
   
   if (!BSE_IS_SERVER (server))
     return BSE_ERROR_PROC_PARAM_INVAL;
 
   file_name = g_strconcat (g_get_home_dir (), "/.bserc", NULL);
   fd = open (file_name,
-#line 497 "bseserver.proc"
+#line 518 "bseserver.proc"
 	     O_WRONLY | O_CREAT | O_TRUNC, 
 	     0666);
   g_free (file_name);
@@ -720,7 +741,7 @@ save_preferences_exec (BseProcedureClass *proc,
   wstore = sfi_wstore_new ();
   sfi_wstore_printf (wstore, "; rc-file for BSE v%s\n", BSE_VERSION);
 
-#line 506 "bseserver.proc"
+#line 527 "bseserver.proc"
   
   sfi_wstore_puts (wstore, "\n; BseGConfig Dump\n");
   rec = bse_gconfig_to_rec (bse_global_config);
@@ -730,7 +751,7 @@ save_preferences_exec (BseProcedureClass *proc,
   sfi_rec_unref (rec);
   sfi_wstore_puts (wstore, "\n");
 
-#line 515 "bseserver.proc"
+#line 536 "bseserver.proc"
   
   sfi_wstore_flush_fd (wstore, fd);
   sfi_wstore_destroy (wstore);
