@@ -95,8 +95,9 @@ static void	    bse_source_real_remove_input	(BseSource	*source,
 							 guint           ochannel);
 static void	    bse_source_real_store_private	(BseObject	*object,
 							 BseStorage	*storage);
-static BseTokenType bse_source_restore_private		(BseObject      *object,
-							 BseStorage     *storage);
+static SfiTokenType bse_source_restore_private		(BseObject      *object,
+							 BseStorage     *storage,
+                                                         GScanner       *scanner);
 static gint	    contexts_compare			(gconstpointer	 bsearch_node1, /* key */
 							 gconstpointer	 bsearch_node2);
 
@@ -130,10 +131,10 @@ BSE_BUILTIN_TYPE (BseSource)
 
   g_assert (BSE_SOURCE_FLAGS_USHIFT < BSE_OBJECT_FLAGS_MAX_SHIFT);
   
-  return bse_type_register_static (BSE_TYPE_ITEM,
-				   "BseSource",
-				   "Base type for sound sources",
-				   &source_info);
+  return bse_type_register_abstract (BSE_TYPE_ITEM,
+                                     "BseSource",
+                                     "Base type for sound sources",
+                                     &source_info);
 }
 
 static void
@@ -1299,18 +1300,20 @@ bse_source_real_remove_input (BseSource *source,
 }
 
 BseErrorType
-bse_source_unset_input (BseSource *source,
+bse_source_check_input (BseSource *source,
 			guint      ichannel,
 			BseSource *osource,
 			guint      ochannel)
 {
   g_return_val_if_fail (BSE_IS_SOURCE (source), BSE_ERROR_INTERNAL);
   g_return_val_if_fail (BSE_IS_SOURCE (osource), BSE_ERROR_INTERNAL);
-  g_return_val_if_fail (BSE_ITEM (source)->parent == BSE_ITEM (osource)->parent, BSE_ERROR_INTERNAL);
+
+  if (BSE_ITEM (source)->parent != BSE_ITEM (osource)->parent)
+    return BSE_ERROR_SOURCE_PARENT_MISMATCH;
   if (BSE_SOURCE_PREPARED (source))     /* FIXME: check context sets */
     {
       g_return_val_if_fail (BSE_SOURCE_PREPARED (osource), BSE_ERROR_INTERNAL);	/* paranoid, checked parent already */
-      g_return_val_if_fail (BSE_SOURCE_N_CONTEXTS (source) == BSE_SOURCE_N_CONTEXTS (osource), BSE_ERROR_INTERNAL);
+      /* prolly wrong: */ g_return_val_if_fail (BSE_SOURCE_N_CONTEXTS (source) == BSE_SOURCE_N_CONTEXTS (osource), BSE_ERROR_INTERNAL);
     }
   else
     g_return_val_if_fail (!BSE_SOURCE_PREPARED (osource), BSE_ERROR_INTERNAL);
@@ -1322,6 +1325,19 @@ bse_source_unset_input (BseSource *source,
   if (check_jchannel_connection (source, ichannel, osource, ochannel) < 0)
     return BSE_ERROR_SOURCE_NO_SUCH_CONNECTION;
 
+  return BSE_ERROR_NONE;
+}
+
+BseErrorType
+bse_source_unset_input (BseSource *source,
+			guint      ichannel,
+			BseSource *osource,
+			guint      ochannel)
+{
+  BseErrorType error = bse_source_check_input (source, ichannel, osource, ochannel);
+  if (error != BSE_ERROR_NONE)
+    return error;
+
   g_object_ref (source);
   g_object_ref (osource);
   BSE_SOURCE_GET_CLASS (source)->remove_input (source, ichannel, osource, ochannel);
@@ -1438,6 +1454,35 @@ bse_source_clear_ichannels (BseSource *source)
   g_object_unref (source);
 }
 
+void
+bse_source_backup_ichannels_to_undo (BseSource *source)
+{
+  BseUndoStack *ustack;
+
+  g_return_if_fail (BSE_IS_SOURCE (source));
+
+  ustack = bse_item_undo_open (source, "unset-input %s", bse_object_debug_name (source));
+  if (ustack)
+    {
+      guint i, j;
+      for (i = 0; i < BSE_SOURCE_N_ICHANNELS (source); i++)
+        {
+          BseSourceInput *input = BSE_SOURCE_INPUT (source, i);
+          
+          if (BSE_SOURCE_IS_JOINT_ICHANNEL (source, i))
+            for (j = 0; j < input->jdata.n_joints; j++)
+              bse_source_input_backup_to_undo (source, i,
+                                               input->jdata.joints[j].osource,
+                                               input->jdata.joints[j].ochannel);
+          else if (input->idata.osource)
+            bse_source_input_backup_to_undo (source, i,
+                                             input->idata.osource,
+                                             input->idata.ochannel);
+        }
+    }
+  bse_item_undo_close (ustack);
+}
+
 void
 bse_source_clear_ochannels (BseSource *source)
 {
@@ -1488,6 +1533,47 @@ bse_source_clear_ochannels (BseSource *source)
   g_object_unref (source);
 }
 
+void
+bse_source_backup_ochannels_to_undo (BseSource *source)
+{
+  BseUndoStack *ustack;
+
+  g_return_if_fail (BSE_IS_SOURCE (source));
+
+  ustack = bse_item_undo_open (source, "unset-input %s", bse_object_debug_name (source));
+  if (ustack)
+    {
+      GSList *slist, *uniq_outputs = NULL;
+
+      for (slist = source->outputs; slist; slist = slist->next)
+        if (!g_slist_find (uniq_outputs, slist->data))
+          uniq_outputs = g_slist_prepend (uniq_outputs, slist->data);
+      
+      for (slist = uniq_outputs; slist; slist = slist->next)
+        {
+          BseSource *isource = slist->data;
+          guint i;
+          for (i = 0; i < BSE_SOURCE_N_ICHANNELS (isource); i++)
+            {
+              BseSourceInput *input = BSE_SOURCE_INPUT (isource, i);
+              if (BSE_SOURCE_IS_JOINT_ICHANNEL (isource, i))
+                {
+                  guint j;
+                  for (j = 0; j < input->jdata.n_joints; j++)
+                    if (input->jdata.joints[j].osource == source)
+                      bse_source_input_backup_to_undo (isource, i,
+                                                       input->jdata.joints[j].osource,
+                                                       input->jdata.joints[j].ochannel);
+                }
+              else if (input->idata.osource == source)
+                bse_source_input_backup_to_undo (isource, i, source, input->idata.ochannel);
+            }
+        }
+      g_slist_free (uniq_outputs);
+    }
+  bse_item_undo_close (ustack);
+}
+
 static void
 bse_source_real_store_private (BseObject  *object,
 			       BseStorage *storage)
@@ -1525,6 +1611,36 @@ bse_source_real_store_private (BseObject  *object,
     }
 }
 
+void
+bse_source_input_backup_to_undo (BseSource      *source,
+                                 guint           ichannel,
+                                 BseSource      *osource,
+                                 guint           ochannel)
+{
+  BseErrorType error = bse_source_check_input (source, ichannel, osource, ochannel);
+  BseUndoStack *ustack;
+  BseStorage *storage;
+
+  g_return_if_fail (error == BSE_ERROR_NONE);
+
+  ustack = bse_item_undo_open (source, "unset-input %s", bse_object_debug_name (source));
+
+  storage = g_object_new (BSE_TYPE_STORAGE, NULL);
+  bse_storage_prepare_write (storage, BSE_STORAGE_SKIP_DEFAULTS | BSE_STORAGE_DBLOCK_CONTAINED);
+
+  bse_storage_break (storage);
+  bse_storage_printf (storage,
+                      "(source-input \"%s\" ",
+                      BSE_SOURCE_ICHANNEL_IDENT (source, ichannel));
+  bse_storage_put_item_link (storage, BSE_ITEM (source), BSE_ITEM (osource));
+  bse_storage_printf (storage, " \"%s\")", BSE_SOURCE_OCHANNEL_IDENT (osource, ochannel));
+
+  bse_item_push_undo_storage (BSE_ITEM (source), ustack, storage);
+  g_object_unref (storage);
+
+  bse_item_undo_close (ustack);
+}
+
 typedef struct _DeferredInput DeferredInput;
 struct _DeferredInput
 {
@@ -1584,12 +1700,12 @@ resolve_osource_input (gpointer     data,
   g_free (dinput);
 }
 
-static BseTokenType
+static SfiTokenType
 bse_source_restore_private (BseObject  *object,
-			    BseStorage *storage)
+			    BseStorage *storage,
+                            GScanner   *scanner)
 {
   BseSource *source = BSE_SOURCE (object);
-  GScanner *scanner = storage->scanner;
 
   if (g_scanner_peek_next_token (scanner) == G_TOKEN_IDENTIFIER &&
       bse_string_equals ("source-input", scanner->next_value.v_identifier))
@@ -1627,5 +1743,5 @@ bse_source_restore_private (BseObject  *object,
       return G_TOKEN_NONE;
     }
   else /* chain parent class' handler */
-    return BSE_OBJECT_CLASS (parent_class)->restore_private (object, storage);
+    return BSE_OBJECT_CLASS (parent_class)->restore_private (object, storage, scanner);
 }
