@@ -65,17 +65,13 @@ BSE_BUILTIN_TYPE (BseSubInstrument)
     INSTRUMENT_IMAGE_WIDTH, INSTRUMENT_IMAGE_HEIGHT,
     INSTRUMENT_IMAGE_RLE_PIXEL_DATA,
   };
-  guint type_id;
-  
-  type_id = bse_type_register_static (BSE_TYPE_SUB_OPORT,
-				      "BseSubInstrument",
-				      "Virtual output module for synthesis networks which "
-				      "implement instruments",
-				      &type_info);
-  bse_categories_register_icon ("/Modules/Virtualization/Instrument Output",
-				type_id,
-				&pixdata);
-  return type_id;
+  GType type = bse_type_register_static (BSE_TYPE_SUB_OPORT,
+                                         "BseSubInstrument",
+                                         "Virtual output module for synthesis networks which "
+                                         "implement instruments",
+                                         &type_info);
+  bse_categories_register_icon ("/Modules/Virtualization/Instrument Output", type, &pixdata);
+  return type;
 }
 
 static void
@@ -129,19 +125,23 @@ bse_sub_instrument_reset_names (BseSubInstrument *self)
   name = BSE_SOURCE_ICHANNEL_IDENT (self, 0);
   if (strcmp (oport->output_ports[0], name) != 0 &&
       (!snet || !bse_snet_oport_name_registered (snet, name)))
-    g_object_set (self, "BseSubOPort::out_port_1", name, NULL);
+    g_object_set (self, /* no undo */
+                  "BseSubOPort::out_port_1", name, NULL);
   name = BSE_SOURCE_ICHANNEL_IDENT (self, 1);
   if (strcmp (oport->output_ports[1], name) != 0 &&
       (!snet || !bse_snet_oport_name_registered (snet, name)))
-    g_object_set (self, "BseSubOPort::out_port_2", name, NULL);
+    g_object_set (self, /* no undo */
+                  "BseSubOPort::out_port_2", name, NULL);
   name = BSE_SOURCE_ICHANNEL_IDENT (self, 2);
   if (strcmp (oport->output_ports[2], name) != 0 &&
       (!snet || !bse_snet_oport_name_registered (snet, name)))
-    g_object_set (self, "BseSubOPort::out_port_3", name, NULL);
+    g_object_set (self, /* no undo */
+                  "BseSubOPort::out_port_3", name, NULL);
   name = BSE_SOURCE_ICHANNEL_IDENT (self, 3);
   if (strcmp (oport->output_ports[3], name) != 0 &&
       (!snet || !bse_snet_oport_name_registered (snet, name)))
-    g_object_set (self, "BseSubOPort::out_port_4", name, NULL);
+    g_object_set (self, /* no undo */
+                  "BseSubOPort::out_port_4", name, NULL);
   g_object_thaw_notify (G_OBJECT (self));
 }
 
