@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1998-1999, 2000-2002 Tim Janik
+ * Copyright (C) 1998-1999, 2000-2003 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,10 +70,6 @@ struct _BseSuperClass
 };
 
 
-/* --- prototypes --- */
-BseProject*     bse_super_get_project		(BseSuper	*super);
-
-
 G_END_DECLS
 
 #endif /* __BSE_SUPER_H__ */
