@@ -29,7 +29,7 @@ extern "C" {
 
 
 /* --- structures --- */
-#define GSL_DATA_HANDLE_PEEK_BUFFER     (8192)
+#define GSL_DATA_HANDLE_PEEK_BUFFER     (2048)
 typedef struct
 {
   gint    dir;   /* initialize direction to -1 or +1 (or 0 for random access) */
@@ -43,7 +43,7 @@ typedef struct
   GslLong tail_cut;
   GslLong min_loop;
   GslLong max_loop;
-} GslLoopSpec;	/* rename this to ...Data... */
+} GslLoopSpec;	/* rename this to GslData... */
 
 
 /* --- data utils --- */
@@ -69,15 +69,26 @@ GslLong		gsl_data_find_block		(GslDataHandle		*handle,
 static inline gfloat gsl_data_handle_peek_value	(GslDataHandle		*dhandle,
 						 GslLong		 position,
 						 GslDataPeekBuffer	*peekbuf);
-gint /* -errno */    gsl_data_handle_dump	(GslDataHandle		*dhandle,
+gint /* errno */     gsl_data_handle_dump	(GslDataHandle		*dhandle,
 						 gint			 fd,
 						 GslWaveFormatType	 format,
 						 guint			 byte_order);
-gint /* -errno */    gsl_data_handle_dump_wav	(GslDataHandle		*dhandle,
+gint /* errno */     gsl_wave_file_dump_header	(gint			 fd,
+						 guint			 n_data_bytes,
+						 guint			 n_bits,
+						 guint			 n_channels,
+						 guint		         sample_freq);
+gint /* errno */     gsl_wave_file_patch_length (gint			 fd,
+						 guint			 n_data_bytes);
+gint /* errno */     gsl_data_handle_dump_wav	(GslDataHandle		*dhandle,
 						 gint			 fd,
 						 guint			 n_bits,
 						 guint			 n_channels,
 						 guint			 sample_freq);
+gint /* errno */     gsl_wave_file_dump_data	(gint			 fd,
+						 guint			 n_bits,
+						 guint			 n_values,
+						 const gfloat		*values);
 void		     gsl_data_handle_dump_wstore(GslDataHandle		*dhandle,
 						 SfiWStore		*wstore,
 						 GslWaveFormatType	 format,
