@@ -4,6 +4,7 @@
 /*-------- begin ../sfi/sfidl generated code --------*/
 
 
+#include <string.h>
 BseIntSeq*
 bse_int_seq_new (void)
 {
@@ -1477,6 +1478,104 @@ bse_song_timing_free (BseSongTiming* rec)
 }
 
 
+BseSampleFileInfo*
+bse_sample_file_info_new (void)
+{
+  BseSampleFileInfo* rec = g_new0 (BseSampleFileInfo, 1);
+  rec->waves = bse_string_seq_new ();
+  return rec;
+}
+
+BseSampleFileInfo*
+bse_sample_file_info_copy_shallow (BseSampleFileInfo* rec)
+{
+  BseSampleFileInfo* rec_copy = NULL;
+
+  g_return_val_if_fail (rec != NULL, NULL);
+  rec_copy = bse_sample_file_info_new ();
+  rec_copy->file = g_strdup (rec->file);
+  rec_copy->size = rec->size;
+  rec_copy->mtime = rec->mtime;
+  rec_copy->loader = g_strdup (rec->loader);
+  rec_copy->waves = bse_string_seq_copy_shallow (rec->waves);
+  rec_copy->error = rec->error;
+  return rec_copy;
+}
+
+BseSampleFileInfo*
+bse_sample_file_info_from_rec (SfiRec *sfi_rec)
+{
+  GValue *element;
+  BseSampleFileInfo* rec;
+
+  g_return_val_if_fail (sfi_rec != NULL, NULL);
+
+  rec = g_new0 (BseSampleFileInfo, 1);
+  element = sfi_rec_get (sfi_rec, "file");
+  if (element)
+    rec->file = g_strdup (sfi_value_get_string (element));
+  element = sfi_rec_get (sfi_rec, "size");
+  if (element)
+    rec->size = sfi_value_get_int (element);
+  element = sfi_rec_get (sfi_rec, "mtime");
+  if (element)
+    rec->mtime = sfi_value_get_num (element);
+  element = sfi_rec_get (sfi_rec, "loader");
+  if (element)
+    rec->loader = g_strdup (sfi_value_get_string (element));
+  element = sfi_rec_get (sfi_rec, "waves");
+  if (element)
+    rec->waves = bse_string_seq_from_seq (sfi_value_get_seq (element));
+  else
+    rec->waves = bse_string_seq_new ();
+  element = sfi_rec_get (sfi_rec, "error");
+  if (element)
+    rec->error = sfi_value_get_int (element);
+  return rec;
+}
+
+SfiRec *
+bse_sample_file_info_to_rec (BseSampleFileInfo* rec)
+{
+  SfiRec *sfi_rec;
+  GValue *element;
+
+  g_return_val_if_fail (rec != NULL, NULL);
+
+  sfi_rec = sfi_rec_new ();
+  element = sfi_value_string (rec->file);
+  sfi_rec_set (sfi_rec, "file", element);
+  sfi_value_free (element);
+  element = sfi_value_int (rec->size);
+  sfi_rec_set (sfi_rec, "size", element);
+  sfi_value_free (element);
+  element = sfi_value_num (rec->mtime);
+  sfi_rec_set (sfi_rec, "mtime", element);
+  sfi_value_free (element);
+  element = sfi_value_string (rec->loader);
+  sfi_rec_set (sfi_rec, "loader", element);
+  sfi_value_free (element);
+  element = sfi_value_seq_take_ref (bse_string_seq_to_seq (rec->waves));
+  sfi_rec_set (sfi_rec, "waves", element);
+  sfi_value_free (element);
+  element = sfi_value_int (rec->error);
+  sfi_rec_set (sfi_rec, "error", element);
+  sfi_value_free (element);
+  return sfi_rec;
+}
+
+void
+bse_sample_file_info_free (BseSampleFileInfo* rec)
+{
+  g_return_if_fail (rec != NULL);
+  
+  g_free (rec->file);
+  g_free (rec->loader);
+  bse_string_seq_free (rec->waves);
+  g_free (rec);
+}
+
+
 BseGConfig*
 bse_gconfig_new (void)
 {
@@ -1601,79 +1700,71 @@ bse_user_msg_type_from_choice (const gchar *choice)
   return choice ? sfi_constants_get_index (G_N_ELEMENTS (bse_user_msg_type_vals), bse_user_msg_type_vals, choice) : 0;
 }
 
-static const SfiConstants bse_error_type_vals[66] = {
-  { "bse-error-no-data", 17, 19 },
-  { "bse-error-device-async", 22, 28 },
-  { "bse-error-pos-alloc", 19, 58 },
-  { "bse-error-file-read-failed", 26, 11 },
-  { "bse-error-file-write-failed", 27, 12 },
-  { "bse-error-file-seek-failed", 26, 10 },
-  { "bse-error-file-open-failed", 26, 9 },
-  { "bse-error-date-cluttered", 24, 34 },
-  { "bse-error-source-channels-connected", 35, 47 },
-  { "bse-error-unimplemented", 23, 26 },
-  { "bse-error-source-type-invalid", 29, 50 },
-  { "bse-error-date-invalid", 22, 33 },
-  { "bse-error-format-invalid", 24, 21 },
-  { "bse-error-date-too-old", 22, 41 },
-  { "bse-error-proc-not-found", 24, 51 },
+static const SfiConstants bse_error_type_vals[58] = {
+  { "bse-error-no-data", 17, 20 },
+  { "bse-error-device-async", 22, 29 },
+  { "bse-error-file-read-failed", 26, 12 },
+  { "bse-error-file-write-failed", 27, 13 },
+  { "bse-error-file-seek-failed", 26, 11 },
+  { "bse-error-file-open-failed", 26, 10 },
+  { "bse-error-source-channels-connected", 35, 39 },
+  { "bse-error-unimplemented", 23, 27 },
+  { "bse-error-source-type-invalid", 29, 43 },
+  { "bse-error-format-invalid", 24, 22 },
+  { "bse-error-proc-not-found", 24, 44 },
   { "bse-error-file-not-found", 24, 8 },
-  { "bse-error-wave-not-found", 24, 24 },
-  { "bse-error-no-space", 18, 15 },
-  { "bse-error-device-not-available", 30, 27 },
-  { "bse-error-source-no-such-module", 31, 42 },
+  { "bse-error-wave-not-found", 24, 25 },
+  { "bse-error-no-space", 18, 16 },
+  { "bse-error-device-not-available", 30, 28 },
+  { "bse-error-source-no-such-module", 31, 34 },
   { "bse-error-none", 14, 0 },
-  { "bse-error-codec-failure", 23, 25 },
-  { "bse-error-source-ichannel-in-use", 32, 46 },
+  { "bse-error-codec-failure", 23, 26 },
+  { "bse-error-source-ichannel-in-use", 32, 38 },
   { "bse-error-file-eof", 18, 7 },
-  { "bse-error-device-caps-mismatch", 30, 31 },
-  { "bse-error-source-bad-loopback", 29, 48 },
+  { "bse-error-device-caps-mismatch", 30, 32 },
+  { "bse-error-source-parent-mismatch", 32, 40 },
+  { "bse-error-source-bad-loopback", 29, 41 },
   { "bse-error-internal", 18, 1 },
-  { "bse-error-proc-param-inval", 26, 53 },
-  { "bse-error-source-no-such-ichannel", 33, 43 },
-  { "bse-error-source-no-such-ochannel", 33, 44 },
-  { "bse-error-invalid-duration", 26, 64 },
-  { "bse-error-source-no-such-connection", 35, 45 },
-  { "bse-error-proc-execution", 24, 54 },
-  { "bse-error-spawn", 15, 57 },
+  { "bse-error-proc-param-inval", 26, 46 },
+  { "bse-error-source-no-such-ichannel", 33, 35 },
+  { "bse-error-source-no-such-ochannel", 33, 36 },
+  { "bse-error-invalid-duration", 26, 56 },
+  { "bse-error-source-no-such-connection", 35, 37 },
+  { "bse-error-proc-execution", 24, 47 },
+  { "bse-error-spawn", 15, 50 },
   { "bse-error-unknown", 17, 2 },
-  { "bse-error-format-unknown", 24, 22 },
-  { "bse-error-no-seek-info", 22, 18 },
+  { "bse-error-format-unknown", 24, 23 },
+  { "bse-error-no-seek-info", 22, 19 },
   { "bse-error-file-io", 17, 3 },
-  { "bse-error-invalid-overlap", 25, 65 },
-  { "bse-error-temp", 14, 23 },
-  { "bse-error-no-header", 19, 17 },
-  { "bse-error-not-owner", 19, 62 },
-  { "bse-error-parse-error", 21, 56 },
-  { "bse-error-date-second-bounds", 28, 40 },
-  { "bse-error-date-minute-bounds", 28, 39 },
-  { "bse-error-date-month-bounds", 27, 36 },
-  { "bse-error-date-year-bounds", 26, 35 },
-  { "bse-error-date-hour-bounds", 26, 38 },
-  { "bse-error-date-day-bounds", 25, 37 },
-  { "bse-error-no-files", 18, 14 },
-  { "bse-error-many-files", 20, 13 },
+  { "bse-error-invalid-overlap", 25, 57 },
+  { "bse-error-temp", 14, 24 },
+  { "bse-error-no-header", 19, 18 },
+  { "bse-error-not-owner", 19, 54 },
+  { "bse-error-file-is-dir", 21, 9 },
+  { "bse-error-parse-error", 21, 49 },
+  { "bse-error-no-files", 18, 15 },
+  { "bse-error-many-files", 20, 14 },
   { "bse-error-file-perms", 20, 4 },
-  { "bse-error-device-get-caps", 25, 30 },
-  { "bse-error-device-set-caps", 25, 32 },
+  { "bse-error-device-get-caps", 25, 31 },
+  { "bse-error-device-set-caps", 25, 33 },
   { "bse-error-file-exists", 21, 6 },
-  { "bse-error-no-target", 19, 61 },
-  { "bse-error-invalid-offset", 24, 63 },
-  { "bse-error-no-event", 18, 60 },
-  { "bse-error-data-corrupt", 22, 20 },
-  { "bse-error-proc-abort", 20, 55 },
-  { "bse-error-no-memory", 19, 16 },
-  { "bse-error-no-entry", 18, 59 },
-  { "bse-error-proc-busy", 19, 52 },
-  { "bse-error-device-busy", 21, 29 },
-  { "bse-error-source-busy", 21, 49 },
+  { "bse-error-no-target", 19, 53 },
+  { "bse-error-invalid-offset", 24, 55 },
+  { "bse-error-no-event", 18, 52 },
+  { "bse-error-data-corrupt", 22, 21 },
+  { "bse-error-proc-abort", 20, 48 },
+  { "bse-error-no-memory", 19, 17 },
+  { "bse-error-no-entry", 18, 51 },
+  { "bse-error-proc-busy", 19, 45 },
+  { "bse-error-device-busy", 21, 30 },
+  { "bse-error-source-busy", 21, 42 },
   { "bse-error-file-busy", 19, 5 },
 };
 
 const gchar*
 bse_error_type_to_choice (BseErrorType value)
 {
-  g_return_val_if_fail (value >= 0 && value <= 65, NULL);
+  g_return_val_if_fail (value >= 0 && value <= 57, NULL);
   return sfi_constants_get_name (G_N_ELEMENTS (bse_error_type_vals), bse_error_type_vals, value);
 }
 
@@ -1709,6 +1800,11 @@ bse_item_check_is_a (SfiProxy bse_item_object, const gchar* type_name) {
   return _retval;
 }
 
+void
+bse_item_clear_undo (SfiProxy bse_item_object) {
+  sfi_glue_vcall_void ("BseItem+clear-undo", 'p', bse_item_object, 0);
+}
+
 SfiProxy
 bse_item_common_ancestor (SfiProxy bse_item_object, SfiProxy item2) {
   SfiProxy _retval;
@@ -1801,6 +1897,18 @@ bse_item_get_uname_path (SfiProxy bse_item_object) {
   return _retval;
 }
 
+void
+bse_item_group_undo (SfiProxy bse_item_object, const gchar* name) {
+  sfi_glue_vcall_void ("BseItem+group-undo", 'p', bse_item_object, 's', name, 0);
+}
+
+SfiBool
+bse_item_internal (SfiProxy bse_item_object) {
+  SfiBool _retval;
+  _retval = sfi_glue_vcall_bool ("BseItem+internal", 'p', bse_item_object, 0);
+  return _retval;
+}
+
 BseProxySeq*
 bse_item_list_proxies (SfiProxy bse_item_object, const gchar* property_name) {
   SfiSeq* _retval;
@@ -1811,11 +1919,40 @@ bse_item_list_proxies (SfiProxy bse_item_object, const gchar* property_name) {
   return _retval_conv;
 }
 
+void
+bse_item_redo (SfiProxy bse_item_object) {
+  sfi_glue_vcall_void ("BseItem+redo", 'p', bse_item_object, 0);
+}
+
+SfiInt
+bse_item_redo_depth (SfiProxy bse_item_object) {
+  SfiInt _retval;
+  _retval = sfi_glue_vcall_int ("BseItem+redo-depth", 'p', bse_item_object, 0);
+  return _retval;
+}
+
 void
 bse_item_set_name (SfiProxy bse_item_object, const gchar* name) {
   sfi_glue_vcall_void ("BseItem+set-name", 'p', bse_item_object, 's', name, 0);
 }
 
+void
+bse_item_undo (SfiProxy bse_item_object) {
+  sfi_glue_vcall_void ("BseItem+undo", 'p', bse_item_object, 0);
+}
+
+SfiInt
+bse_item_undo_depth (SfiProxy bse_item_object) {
+  SfiInt _retval;
+  _retval = sfi_glue_vcall_int ("BseItem+undo-depth", 'p', bse_item_object, 0);
+  return _retval;
+}
+
+void
+bse_item_ungroup_undo (SfiProxy bse_item_object) {
+  sfi_glue_vcall_void ("BseItem+ungroup-undo", 'p', bse_item_object, 0);
+}
+
 void
 bse_item_unuse (SfiProxy bse_item_object) {
   sfi_glue_vcall_void ("BseItem+unuse", 'p', bse_item_object, 0);
@@ -1950,6 +2087,11 @@ bse_source_set_input_by_id (SfiProxy bse_source_object, SfiInt input_channel, Sf
   return bse_error_type_from_choice (_retval);
 }
 
+void
+bse_source_set_pos (SfiProxy bse_source_object, SfiReal x_pos, SfiReal y_pos) {
+  sfi_glue_vcall_void ("BseSource+set-pos", 'p', bse_source_object, 'r', x_pos, 'r', y_pos, 0);
+}
+
 BseErrorType
 bse_source_unset_input (SfiProxy bse_source_object, const gchar* input_channel, SfiProxy omodule, const gchar* output_channel) {
   const gchar * _retval;
@@ -2107,17 +2249,22 @@ bse_project_can_play (SfiProxy bse_project_object) {
   return _retval;
 }
 
+void
+bse_project_clear_undo (SfiProxy bse_project_object) {
+  sfi_glue_vcall_void ("BseProject+clear-undo", 'p', bse_project_object, 0);
+}
+
 SfiProxy
-bse_project_create_midi_synth (SfiProxy bse_project_object, const gchar* name) {
+bse_project_create_csynth (SfiProxy bse_project_object, const gchar* name) {
   SfiProxy _retval;
-  _retval = sfi_glue_vcall_proxy ("BseProject+create-midi-synth", 'p', bse_project_object, 's', name, 0);
+  _retval = sfi_glue_vcall_proxy ("BseProject+create-csynth", 'p', bse_project_object, 's', name, 0);
   return _retval;
 }
 
 SfiProxy
-bse_project_create_snet (SfiProxy bse_project_object, const gchar* name) {
+bse_project_create_midi_synth (SfiProxy bse_project_object, const gchar* name) {
   SfiProxy _retval;
-  _retval = sfi_glue_vcall_proxy ("BseProject+create-snet", 'p', bse_project_object, 's', name, 0);
+  _retval = sfi_glue_vcall_proxy ("BseProject+create-midi-synth", 'p', bse_project_object, 's', name, 0);
   return _retval;
 }
 
@@ -2133,13 +2280,6 @@ bse_project_deactivate (SfiProxy bse_project_object) {
   sfi_glue_vcall_void ("BseProject+deactivate", 'p', bse_project_object, 0);
 }
 
-SfiProxy
-bse_project_ensure_wave_repo (SfiProxy bse_project_object) {
-  SfiProxy _retval;
-  _retval = sfi_glue_vcall_proxy ("BseProject+ensure-wave-repo", 'p', bse_project_object, 0);
-  return _retval;
-}
-
 SfiProxy
 bse_project_find_item (SfiProxy bse_project_object, const gchar* uname_path) {
   SfiProxy _retval;
@@ -2171,6 +2311,20 @@ bse_project_get_supers (SfiProxy bse_project_object) {
   return _retval_conv;
 }
 
+SfiProxy
+bse_project_get_wave_repo (SfiProxy bse_project_object) {
+  SfiProxy _retval;
+  _retval = sfi_glue_vcall_proxy ("BseProject+get-wave-repo", 'p', bse_project_object, 0);
+  return _retval;
+}
+
+SfiBool
+bse_project_is_active (SfiProxy bse_project_object) {
+  SfiBool _retval;
+  _retval = sfi_glue_vcall_bool ("BseProject+is-active", 'p', bse_project_object, 0);
+  return _retval;
+}
+
 SfiBool
 bse_project_is_playing (SfiProxy bse_project_object) {
   SfiBool _retval;
@@ -2205,6 +2359,23 @@ bse_project_play (SfiProxy bse_project_object) {
   return bse_error_type_from_choice (_retval);
 }
 
+void
+bse_project_redo (SfiProxy bse_project_object) {
+  sfi_glue_vcall_void ("BseProject+redo", 'p', bse_project_object, 0);
+}
+
+SfiInt
+bse_project_redo_depth (SfiProxy bse_project_object) {
+  SfiInt _retval;
+  _retval = sfi_glue_vcall_int ("BseProject+redo-depth", 'p', bse_project_object, 0);
+  return _retval;
+}
+
+void
+bse_project_remove_snet (SfiProxy bse_project_object, SfiProxy snet) {
+  sfi_glue_vcall_void ("BseProject+remove-snet", 'p', bse_project_object, 'p', snet, 0);
+}
+
 void
 bse_project_start_playback (SfiProxy bse_project_object) {
   sfi_glue_vcall_void ("BseProject+start-playback", 'p', bse_project_object, 0);
@@ -2220,6 +2391,18 @@ bse_project_stop_playback (SfiProxy bse_project_object) {
   sfi_glue_vcall_void ("BseProject+stop-playback", 'p', bse_project_object, 0);
 }
 
+void
+bse_project_undo (SfiProxy bse_project_object) {
+  sfi_glue_vcall_void ("BseProject+undo", 'p', bse_project_object, 0);
+}
+
+SfiInt
+bse_project_undo_depth (SfiProxy bse_project_object) {
+  SfiInt _retval;
+  _retval = sfi_glue_vcall_int ("BseProject+undo-depth", 'p', bse_project_object, 0);
+  return _retval;
+}
+
 SfiBool
 bse_server_can_load (SfiProxy bse_server_object, const gchar* file_name) {
   SfiBool _retval;
@@ -2320,6 +2503,11 @@ bse_wave_osc_request_pcm_position (SfiProxy bse_wave_osc_object) {
   sfi_glue_vcall_void ("BseWaveOsc+request-pcm-position", 'p', bse_wave_osc_object, 0);
 }
 
+void
+bse_wave_osc_set_from_editable_sample (SfiProxy bse_wave_osc_object, SfiProxy esample) {
+  sfi_glue_vcall_void ("BseWaveOsc+set-from-editable-sample", 'p', bse_wave_osc_object, 'p', esample, 0);
+}
+
 SfiReal
 bse_wave_chunk_get_mix_freq (SfiProxy bse_wave_object, SfiInt chunk_index) {
   SfiReal _retval;
@@ -2379,11 +2567,11 @@ bse_track_get_timing (SfiProxy bse_track_object, SfiInt tick) {
   return _retval_conv;
 }
 
-BseErrorType
+SfiInt
 bse_track_insert_part (SfiProxy bse_track_object, SfiInt tick, SfiProxy part) {
-  const gchar * _retval;
-  _retval = sfi_glue_vcall_choice ("BseTrack+insert-part", 'p', bse_track_object, 'i', tick, 'p', part, 0);
-  return bse_error_type_from_choice (_retval);
+  SfiInt _retval;
+  _retval = sfi_glue_vcall_int ("BseTrack+insert-part", 'p', bse_track_object, 'i', tick, 'p', part, 0);
+  return _retval;
 }
 
 BseTrackPartSeq*
@@ -2396,6 +2584,11 @@ bse_track_list_parts (SfiProxy bse_track_object) {
   return _retval_conv;
 }
 
+void
+bse_track_remove_link (SfiProxy bse_track_object, SfiInt id) {
+  sfi_glue_vcall_void ("BseTrack+remove-link", 'p', bse_track_object, 'i', id, 0);
+}
+
 void
 bse_track_remove_tick (SfiProxy bse_track_object, SfiInt tick) {
   sfi_glue_vcall_void ("BseTrack+remove-tick", 'p', bse_track_object, 'i', tick, 0);
@@ -2777,6 +2970,16 @@ bse_categories_match (const gchar* pattern) {
   return _retval_conv;
 }
 
+BseCategorySeq*
+bse_categories_match_method (const gchar* pattern, const gchar* type) {
+  SfiSeq* _retval;
+  BseCategorySeq* _retval_conv;
+  _retval = sfi_glue_vcall_seq ("bse-categories-match-method", 's', pattern, 's', type, 0);
+  _retval_conv = bse_category_seq_from_seq (_retval);
+  if (_retval_conv != NULL) sfi_glue_gc_add (_retval_conv, bse_category_seq_free);
+  return _retval_conv;
+}
+
 BseCategorySeq*
 bse_categories_match_typed (const gchar* pattern, const gchar* type) {
   SfiSeq* _retval;
@@ -2822,6 +3025,23 @@ bse_note_to_freq (SfiInt note, SfiInt fine_tune) {
   return _retval;
 }
 
+BseSampleFileInfo*
+bse_sample_file_info (const gchar* file_name) {
+  SfiRec* _retval;
+  BseSampleFileInfo* _retval_conv;
+  _retval = sfi_glue_vcall_rec ("bse-sample-file-info", 's', file_name, 0);
+  _retval_conv = bse_sample_file_info_from_rec (_retval);
+  if (_retval_conv != NULL) sfi_glue_gc_add (_retval_conv, bse_sample_file_info_free);
+  return _retval_conv;
+}
+
+SfiReal
+bse_string_extract_number (const gchar* string, const gchar* format, SfiReal aux_base, SfiReal dflt) {
+  SfiReal _retval;
+  _retval = sfi_glue_vcall_real ("bse-string-extract-number", 's', string, 's', format, 'r', aux_base, 'r', dflt, 0);
+  return _retval;
+}
+
 const gchar*
 bse_type_blurb (const gchar* type) {
   const gchar* _retval;
