@@ -101,6 +101,11 @@
 #define bse_song_timing_from_rec foo_prefix_bse_song_timing_from_rec
 #define bse_song_timing_to_rec foo_prefix_bse_song_timing_to_rec
 #define bse_song_timing_free foo_prefix_bse_song_timing_free
+#define bse_sample_file_info_new foo_prefix_bse_sample_file_info_new
+#define bse_sample_file_info_copy_shallow foo_prefix_bse_sample_file_info_copy_shallow
+#define bse_sample_file_info_from_rec foo_prefix_bse_sample_file_info_from_rec
+#define bse_sample_file_info_to_rec foo_prefix_bse_sample_file_info_to_rec
+#define bse_sample_file_info_free foo_prefix_bse_sample_file_info_free
 #define bse_gconfig_new foo_prefix_bse_gconfig_new
 #define bse_gconfig_copy_shallow foo_prefix_bse_gconfig_copy_shallow
 #define bse_gconfig_from_rec foo_prefix_bse_gconfig_from_rec
@@ -115,6 +120,7 @@
 #define bse_project_state_to_choice foo_prefix_bse_project_state_to_choice
 #define bse_project_state_from_choice foo_prefix_bse_project_state_from_choice
 #define bse_item_check_is_a foo_prefix_bse_item_check_is_a
+#define bse_item_clear_undo foo_prefix_bse_item_clear_undo
 #define bse_item_common_ancestor foo_prefix_bse_item_common_ancestor
 #define bse_item_fixme_get_parasite foo_prefix_bse_item_fixme_get_parasite
 #define bse_item_fixme_set_parasite foo_prefix_bse_item_fixme_set_parasite
@@ -128,8 +134,15 @@
 #define bse_item_get_type_blurb foo_prefix_bse_item_get_type_blurb
 #define bse_item_get_type_name foo_prefix_bse_item_get_type_name
 #define bse_item_get_uname_path foo_prefix_bse_item_get_uname_path
+#define bse_item_group_undo foo_prefix_bse_item_group_undo
+#define bse_item_internal foo_prefix_bse_item_internal
 #define bse_item_list_proxies foo_prefix_bse_item_list_proxies
+#define bse_item_redo foo_prefix_bse_item_redo
+#define bse_item_redo_depth foo_prefix_bse_item_redo_depth
 #define bse_item_set_name foo_prefix_bse_item_set_name
+#define bse_item_undo foo_prefix_bse_item_undo
+#define bse_item_undo_depth foo_prefix_bse_item_undo_depth
+#define bse_item_ungroup_undo foo_prefix_bse_item_ungroup_undo
 #define bse_item_unuse foo_prefix_bse_item_unuse
 #define bse_item_use foo_prefix_bse_item_use
 #define bse_source_clear_inputs foo_prefix_bse_source_clear_inputs
@@ -150,6 +163,7 @@
 #define bse_source_ochannel_name foo_prefix_bse_source_ochannel_name
 #define bse_source_set_input foo_prefix_bse_source_set_input
 #define bse_source_set_input_by_id foo_prefix_bse_source_set_input_by_id
+#define bse_source_set_pos foo_prefix_bse_source_set_pos
 #define bse_source_unset_input foo_prefix_bse_source_unset_input
 #define bse_source_unset_input_by_id foo_prefix_bse_source_unset_input_by_id
 #define bse_container_get_item foo_prefix_bse_container_get_item
@@ -173,22 +187,29 @@
 #define bse_project_activate foo_prefix_bse_project_activate
 #define bse_project_auto_deactivate foo_prefix_bse_project_auto_deactivate
 #define bse_project_can_play foo_prefix_bse_project_can_play
+#define bse_project_clear_undo foo_prefix_bse_project_clear_undo
+#define bse_project_create_csynth foo_prefix_bse_project_create_csynth
 #define bse_project_create_midi_synth foo_prefix_bse_project_create_midi_synth
-#define bse_project_create_snet foo_prefix_bse_project_create_snet
 #define bse_project_create_song foo_prefix_bse_project_create_song
 #define bse_project_deactivate foo_prefix_bse_project_deactivate
-#define bse_project_ensure_wave_repo foo_prefix_bse_project_ensure_wave_repo
 #define bse_project_find_item foo_prefix_bse_project_find_item
 #define bse_project_get_data_pocket foo_prefix_bse_project_get_data_pocket
 #define bse_project_get_state foo_prefix_bse_project_get_state
 #define bse_project_get_supers foo_prefix_bse_project_get_supers
+#define bse_project_get_wave_repo foo_prefix_bse_project_get_wave_repo
+#define bse_project_is_active foo_prefix_bse_project_is_active
 #define bse_project_is_playing foo_prefix_bse_project_is_playing
 #define bse_project_list_uname_paths foo_prefix_bse_project_list_uname_paths
 #define bse_project_match_items_by_uname foo_prefix_bse_project_match_items_by_uname
 #define bse_project_play foo_prefix_bse_project_play
+#define bse_project_redo foo_prefix_bse_project_redo
+#define bse_project_redo_depth foo_prefix_bse_project_redo_depth
+#define bse_project_remove_snet foo_prefix_bse_project_remove_snet
 #define bse_project_start_playback foo_prefix_bse_project_start_playback
 #define bse_project_stop foo_prefix_bse_project_stop
 #define bse_project_stop_playback foo_prefix_bse_project_stop_playback
+#define bse_project_undo foo_prefix_bse_project_undo
+#define bse_project_undo_depth foo_prefix_bse_project_undo_depth
 #define bse_server_can_load foo_prefix_bse_server_can_load
 #define bse_server_construct_note foo_prefix_bse_server_construct_note
 #define bse_server_describe_note foo_prefix_bse_server_describe_note
@@ -203,6 +224,7 @@
 #define bse_server_use_new_project foo_prefix_bse_server_use_new_project
 #define bse_wave_osc_pcm_seek_perc foo_prefix_bse_wave_osc_pcm_seek_perc
 #define bse_wave_osc_request_pcm_position foo_prefix_bse_wave_osc_request_pcm_position
+#define bse_wave_osc_set_from_editable_sample foo_prefix_bse_wave_osc_set_from_editable_sample
 #define bse_wave_chunk_get_mix_freq foo_prefix_bse_wave_chunk_get_mix_freq
 #define bse_wave_chunk_get_osc_freq foo_prefix_bse_wave_chunk_get_osc_freq
 #define bse_wave_load_wave foo_prefix_bse_wave_load_wave
@@ -213,6 +235,7 @@
 #define bse_track_get_timing foo_prefix_bse_track_get_timing
 #define bse_track_insert_part foo_prefix_bse_track_insert_part
 #define bse_track_list_parts foo_prefix_bse_track_list_parts
+#define bse_track_remove_link foo_prefix_bse_track_remove_link
 #define bse_track_remove_tick foo_prefix_bse_track_remove_tick
 #define bse_part_change_note foo_prefix_bse_part_change_note
 #define bse_part_check_overlap foo_prefix_bse_part_check_overlap
@@ -269,12 +292,15 @@
 #define bse_randomizer foo_prefix_bse_randomizer
 #define bse_category_from_id foo_prefix_bse_category_from_id
 #define bse_categories_match foo_prefix_bse_categories_match
+#define bse_categories_match_method foo_prefix_bse_categories_match_method
 #define bse_categories_match_typed foo_prefix_bse_categories_match_typed
 #define bse_error_blurb foo_prefix_bse_error_blurb
 #define bse_error_name foo_prefix_bse_error_name
 #define bse_proxy_check foo_prefix_bse_proxy_check
 #define bse_note_from_freq foo_prefix_bse_note_from_freq
 #define bse_note_to_freq foo_prefix_bse_note_to_freq
+#define bse_sample_file_info foo_prefix_bse_sample_file_info
+#define bse_string_extract_number foo_prefix_bse_string_extract_number
 #define bse_type_blurb foo_prefix_bse_type_blurb
 
 typedef struct _BseIntSeq BseIntSeq;
@@ -293,6 +319,7 @@ typedef struct _BseCategory BseCategory;
 typedef struct _BseDot BseDot;
 typedef struct _BseTrackPart BseTrackPart;
 typedef struct _BseSongTiming BseSongTiming;
+typedef struct _BseSampleFileInfo BseSampleFileInfo;
 typedef struct _BseGConfig BseGConfig;
 
 typedef enum {
@@ -322,63 +349,55 @@ typedef enum {
   BSE_ERROR_FILE_EXISTS = 6,
   BSE_ERROR_FILE_EOF = 7,
   BSE_ERROR_FILE_NOT_FOUND = 8,
-  BSE_ERROR_FILE_OPEN_FAILED = 9,
-  BSE_ERROR_FILE_SEEK_FAILED = 10,
-  BSE_ERROR_FILE_READ_FAILED = 11,
-  BSE_ERROR_FILE_WRITE_FAILED = 12,
-  BSE_ERROR_MANY_FILES = 13,
-  BSE_ERROR_NO_FILES = 14,
-  BSE_ERROR_NO_SPACE = 15,
-  BSE_ERROR_NO_MEMORY = 16,
-  BSE_ERROR_NO_HEADER = 17,
-  BSE_ERROR_NO_SEEK_INFO = 18,
-  BSE_ERROR_NO_DATA = 19,
-  BSE_ERROR_DATA_CORRUPT = 20,
-  BSE_ERROR_FORMAT_INVALID = 21,
-  BSE_ERROR_FORMAT_UNKNOWN = 22,
-  BSE_ERROR_TEMP = 23,
-  BSE_ERROR_WAVE_NOT_FOUND = 24,
-  BSE_ERROR_CODEC_FAILURE = 25,
-  BSE_ERROR_UNIMPLEMENTED = 26,
-  BSE_ERROR_DEVICE_NOT_AVAILABLE = 27,
-  BSE_ERROR_DEVICE_ASYNC = 28,
-  BSE_ERROR_DEVICE_BUSY = 29,
-  BSE_ERROR_DEVICE_GET_CAPS = 30,
-  BSE_ERROR_DEVICE_CAPS_MISMATCH = 31,
-  BSE_ERROR_DEVICE_SET_CAPS = 32,
-  BSE_ERROR_DATE_INVALID = 33,
-  BSE_ERROR_DATE_CLUTTERED = 34,
-  BSE_ERROR_DATE_YEAR_BOUNDS = 35,
-  BSE_ERROR_DATE_MONTH_BOUNDS = 36,
-  BSE_ERROR_DATE_DAY_BOUNDS = 37,
-  BSE_ERROR_DATE_HOUR_BOUNDS = 38,
-  BSE_ERROR_DATE_MINUTE_BOUNDS = 39,
-  BSE_ERROR_DATE_SECOND_BOUNDS = 40,
-  BSE_ERROR_DATE_TOO_OLD = 41,
-  BSE_ERROR_SOURCE_NO_SUCH_MODULE = 42,
-  BSE_ERROR_SOURCE_NO_SUCH_ICHANNEL = 43,
-  BSE_ERROR_SOURCE_NO_SUCH_OCHANNEL = 44,
-  BSE_ERROR_SOURCE_NO_SUCH_CONNECTION = 45,
-  BSE_ERROR_SOURCE_ICHANNEL_IN_USE = 46,
-  BSE_ERROR_SOURCE_CHANNELS_CONNECTED = 47,
-  BSE_ERROR_SOURCE_BAD_LOOPBACK = 48,
-  BSE_ERROR_SOURCE_BUSY = 49,
-  BSE_ERROR_SOURCE_TYPE_INVALID = 50,
-  BSE_ERROR_PROC_NOT_FOUND = 51,
-  BSE_ERROR_PROC_BUSY = 52,
-  BSE_ERROR_PROC_PARAM_INVAL = 53,
-  BSE_ERROR_PROC_EXECUTION = 54,
-  BSE_ERROR_PROC_ABORT = 55,
-  BSE_ERROR_PARSE_ERROR = 56,
-  BSE_ERROR_SPAWN = 57,
-  BSE_ERROR_POS_ALLOC = 58,
-  BSE_ERROR_NO_ENTRY = 59,
-  BSE_ERROR_NO_EVENT = 60,
-  BSE_ERROR_NO_TARGET = 61,
-  BSE_ERROR_NOT_OWNER = 62,
-  BSE_ERROR_INVALID_OFFSET = 63,
-  BSE_ERROR_INVALID_DURATION = 64,
-  BSE_ERROR_INVALID_OVERLAP = 65,
+  BSE_ERROR_FILE_IS_DIR = 9,
+  BSE_ERROR_FILE_OPEN_FAILED = 10,
+  BSE_ERROR_FILE_SEEK_FAILED = 11,
+  BSE_ERROR_FILE_READ_FAILED = 12,
+  BSE_ERROR_FILE_WRITE_FAILED = 13,
+  BSE_ERROR_MANY_FILES = 14,
+  BSE_ERROR_NO_FILES = 15,
+  BSE_ERROR_NO_SPACE = 16,
+  BSE_ERROR_NO_MEMORY = 17,
+  BSE_ERROR_NO_HEADER = 18,
+  BSE_ERROR_NO_SEEK_INFO = 19,
+  BSE_ERROR_NO_DATA = 20,
+  BSE_ERROR_DATA_CORRUPT = 21,
+  BSE_ERROR_FORMAT_INVALID = 22,
+  BSE_ERROR_FORMAT_UNKNOWN = 23,
+  BSE_ERROR_TEMP = 24,
+  BSE_ERROR_WAVE_NOT_FOUND = 25,
+  BSE_ERROR_CODEC_FAILURE = 26,
+  BSE_ERROR_UNIMPLEMENTED = 27,
+  BSE_ERROR_DEVICE_NOT_AVAILABLE = 28,
+  BSE_ERROR_DEVICE_ASYNC = 29,
+  BSE_ERROR_DEVICE_BUSY = 30,
+  BSE_ERROR_DEVICE_GET_CAPS = 31,
+  BSE_ERROR_DEVICE_CAPS_MISMATCH = 32,
+  BSE_ERROR_DEVICE_SET_CAPS = 33,
+  BSE_ERROR_SOURCE_NO_SUCH_MODULE = 34,
+  BSE_ERROR_SOURCE_NO_SUCH_ICHANNEL = 35,
+  BSE_ERROR_SOURCE_NO_SUCH_OCHANNEL = 36,
+  BSE_ERROR_SOURCE_NO_SUCH_CONNECTION = 37,
+  BSE_ERROR_SOURCE_ICHANNEL_IN_USE = 38,
+  BSE_ERROR_SOURCE_CHANNELS_CONNECTED = 39,
+  BSE_ERROR_SOURCE_PARENT_MISMATCH = 40,
+  BSE_ERROR_SOURCE_BAD_LOOPBACK = 41,
+  BSE_ERROR_SOURCE_BUSY = 42,
+  BSE_ERROR_SOURCE_TYPE_INVALID = 43,
+  BSE_ERROR_PROC_NOT_FOUND = 44,
+  BSE_ERROR_PROC_BUSY = 45,
+  BSE_ERROR_PROC_PARAM_INVAL = 46,
+  BSE_ERROR_PROC_EXECUTION = 47,
+  BSE_ERROR_PROC_ABORT = 48,
+  BSE_ERROR_PARSE_ERROR = 49,
+  BSE_ERROR_SPAWN = 50,
+  BSE_ERROR_NO_ENTRY = 51,
+  BSE_ERROR_NO_EVENT = 52,
+  BSE_ERROR_NO_TARGET = 53,
+  BSE_ERROR_NOT_OWNER = 54,
+  BSE_ERROR_INVALID_OFFSET = 55,
+  BSE_ERROR_INVALID_DURATION = 56,
+  BSE_ERROR_INVALID_OVERLAP = 57,
 } BseErrorType;
 const gchar* bse_error_type_to_choice (BseErrorType value);
 BseErrorType bse_error_type_from_choice (const gchar *choice);
@@ -479,6 +498,14 @@ struct _BseSongTiming {
   SfiInt tpqn;
   SfiInt tpt;
 };
+struct _BseSampleFileInfo {
+  gchar* file;
+  SfiInt size;
+  SfiNum mtime;
+  gchar* loader;
+  BseStringSeq* waves;
+  SfiInt error;
+};
 struct _BseGConfig {
   SfiInt synth_block_size;
   SfiInt synth_mixing_freq;
@@ -599,6 +626,12 @@ BseSongTiming* bse_song_timing_from_rec (SfiRec *sfi_rec);
 SfiRec *bse_song_timing_to_rec (BseSongTiming* rec);
 void bse_song_timing_free (BseSongTiming* rec);
 
+BseSampleFileInfo* bse_sample_file_info_new (void);
+BseSampleFileInfo* bse_sample_file_info_copy_shallow (BseSampleFileInfo* rec);
+BseSampleFileInfo* bse_sample_file_info_from_rec (SfiRec *sfi_rec);
+SfiRec *bse_sample_file_info_to_rec (BseSampleFileInfo* rec);
+void bse_sample_file_info_free (BseSampleFileInfo* rec);
+
 BseGConfig* bse_gconfig_new (void);
 BseGConfig* bse_gconfig_copy_shallow (BseGConfig* rec);
 BseGConfig* bse_gconfig_from_rec (SfiRec *sfi_rec);
@@ -618,9 +651,11 @@ extern SfiRecFields bse_category_fields;
 extern SfiRecFields bse_dot_fields;
 extern SfiRecFields bse_track_part_fields;
 extern SfiRecFields bse_song_timing_fields;
+extern SfiRecFields bse_sample_file_info_fields;
 extern SfiRecFields bse_gconfig_fields;
 
 #define BSE_IS_ITEM(proxy) bse_proxy_is_a ((proxy), "BseItem")
+#define BSE_IS_PCM_WRITER(proxy) bse_proxy_is_a ((proxy), "BsePcmWriter")
 #define BSE_IS_SOURCE(proxy) bse_proxy_is_a ((proxy), "BseSource")
 #define BSE_IS_SNOOPER(proxy) bse_proxy_is_a ((proxy), "BseSnooper")
 #define BSE_IS_CONSTANT(proxy) bse_proxy_is_a ((proxy), "BseConstant")
@@ -629,6 +664,7 @@ extern SfiRecFields bse_gconfig_fields;
 #define BSE_IS_WAVE_REPO(proxy) bse_proxy_is_a ((proxy), "BseWaveRepo")
 #define BSE_IS_SNET(proxy) bse_proxy_is_a ((proxy), "BseSNet")
 #define BSE_IS_MIDI_SYNTH(proxy) bse_proxy_is_a ((proxy), "BseMidiSynth")
+#define BSE_IS_CSYNTH(proxy) bse_proxy_is_a ((proxy), "BseCSynth")
 #define BSE_IS_SONG(proxy) bse_proxy_is_a ((proxy), "BseSong")
 #define BSE_IS_PROJECT(proxy) bse_proxy_is_a ((proxy), "BseProject")
 #define BSE_IS_SERVER(proxy) bse_proxy_is_a ((proxy), "BseServer")
@@ -659,6 +695,7 @@ extern SfiRecFields bse_gconfig_fields;
 #define BSE_IS_PCM_DEVICE_OSS(proxy) bse_proxy_is_a ((proxy), "BsePcmDeviceOSS")
 
 SfiBool bse_item_check_is_a (SfiProxy bse_item_object, const gchar* type_name);
+void bse_item_clear_undo (SfiProxy bse_item_object);
 SfiProxy bse_item_common_ancestor (SfiProxy bse_item_object, SfiProxy item2);
 SfiFBlock* bse_item_fixme_get_parasite (SfiProxy bse_item_object, const gchar* name);
 void bse_item_fixme_set_parasite (SfiProxy bse_item_object, const gchar* name, SfiFBlock* parasite);
@@ -672,8 +709,15 @@ const gchar* bse_item_get_type (SfiProxy bse_item_object);
 const gchar* bse_item_get_type_blurb (SfiProxy bse_item_object);
 const gchar* bse_item_get_type_name (SfiProxy bse_item_object);
 const gchar* bse_item_get_uname_path (SfiProxy bse_item_object);
+void bse_item_group_undo (SfiProxy bse_item_object, const gchar* name);
+SfiBool bse_item_internal (SfiProxy bse_item_object);
 BseProxySeq* bse_item_list_proxies (SfiProxy bse_item_object, const gchar* property_name);
+void bse_item_redo (SfiProxy bse_item_object);
+SfiInt bse_item_redo_depth (SfiProxy bse_item_object);
 void bse_item_set_name (SfiProxy bse_item_object, const gchar* name);
+void bse_item_undo (SfiProxy bse_item_object);
+SfiInt bse_item_undo_depth (SfiProxy bse_item_object);
+void bse_item_ungroup_undo (SfiProxy bse_item_object);
 void bse_item_unuse (SfiProxy bse_item_object);
 SfiProxy bse_item_use (SfiProxy bse_item_object);
 void bse_source_clear_inputs (SfiProxy bse_source_object);
@@ -694,6 +738,7 @@ const gchar* bse_source_ochannel_ident (SfiProxy bse_source_object, SfiInt input
 const gchar* bse_source_ochannel_name (SfiProxy bse_source_object, SfiInt input_channel);
 BseErrorType bse_source_set_input (SfiProxy bse_source_object, const gchar* input_channel, SfiProxy omodule, const gchar* output_channel);
 BseErrorType bse_source_set_input_by_id (SfiProxy bse_source_object, SfiInt input_channel, SfiProxy omodule, SfiInt output_channel);
+void bse_source_set_pos (SfiProxy bse_source_object, SfiReal x_pos, SfiReal y_pos);
 BseErrorType bse_source_unset_input (SfiProxy bse_source_object, const gchar* input_channel, SfiProxy omodule, const gchar* output_channel);
 BseErrorType bse_source_unset_input_by_id (SfiProxy bse_source_object, SfiInt input_channel, SfiProxy omodule, SfiInt output_channel);
 SfiProxy bse_container_get_item (SfiProxy bse_container_object, const gchar* item_type, SfiInt seq_id);
@@ -717,22 +762,29 @@ BseErrorType bse_project_store_bse (SfiProxy bse_project_object, const gchar* fi
 BseErrorType bse_project_activate (SfiProxy bse_project_object);
 void bse_project_auto_deactivate (SfiProxy bse_project_object, SfiInt msec_delay);
 SfiBool bse_project_can_play (SfiProxy bse_project_object);
+void bse_project_clear_undo (SfiProxy bse_project_object);
+SfiProxy bse_project_create_csynth (SfiProxy bse_project_object, const gchar* name);
 SfiProxy bse_project_create_midi_synth (SfiProxy bse_project_object, const gchar* name);
-SfiProxy bse_project_create_snet (SfiProxy bse_project_object, const gchar* name);
 SfiProxy bse_project_create_song (SfiProxy bse_project_object, const gchar* name);
 void bse_project_deactivate (SfiProxy bse_project_object);
-SfiProxy bse_project_ensure_wave_repo (SfiProxy bse_project_object);
 SfiProxy bse_project_find_item (SfiProxy bse_project_object, const gchar* uname_path);
 SfiProxy bse_project_get_data_pocket (SfiProxy bse_project_object, const gchar* name);
 BseProjectState bse_project_get_state (SfiProxy bse_project_object);
 BseProxySeq* bse_project_get_supers (SfiProxy bse_project_object);
+SfiProxy bse_project_get_wave_repo (SfiProxy bse_project_object);
+SfiBool bse_project_is_active (SfiProxy bse_project_object);
 SfiBool bse_project_is_playing (SfiProxy bse_project_object);
 BseStringSeq* bse_project_list_uname_paths (SfiProxy bse_project_object, const gchar* item_type);
 BseProxySeq* bse_project_match_items_by_uname (SfiProxy bse_project_object, const gchar* item_type, const gchar* uname);
 BseErrorType bse_project_play (SfiProxy bse_project_object);
+void bse_project_redo (SfiProxy bse_project_object);
+SfiInt bse_project_redo_depth (SfiProxy bse_project_object);
+void bse_project_remove_snet (SfiProxy bse_project_object, SfiProxy snet);
 void bse_project_start_playback (SfiProxy bse_project_object);
 void bse_project_stop (SfiProxy bse_project_object);
 void bse_project_stop_playback (SfiProxy bse_project_object);
+void bse_project_undo (SfiProxy bse_project_object);
+SfiInt bse_project_undo_depth (SfiProxy bse_project_object);
 SfiBool bse_server_can_load (SfiProxy bse_server_object, const gchar* file_name);
 BseNoteDescription* bse_server_construct_note (SfiProxy bse_server_object, SfiInt semitone, SfiInt octave, SfiInt fine_tune);
 BseNoteDescription* bse_server_describe_note (SfiProxy bse_server_object, SfiInt note, SfiInt fine_tune);
@@ -747,6 +799,7 @@ void bse_server_save_preferences (SfiProxy bse_server_object);
 SfiProxy bse_server_use_new_project (SfiProxy bse_server_object, const gchar* name);
 void bse_wave_osc_pcm_seek_perc (SfiProxy bse_wave_osc_object, SfiReal pos_perc);
 void bse_wave_osc_request_pcm_position (SfiProxy bse_wave_osc_object);
+void bse_wave_osc_set_from_editable_sample (SfiProxy bse_wave_osc_object, SfiProxy esample);
 SfiReal bse_wave_chunk_get_mix_freq (SfiProxy bse_wave_object, SfiInt chunk_index);
 SfiReal bse_wave_chunk_get_osc_freq (SfiProxy bse_wave_object, SfiInt chunk_index);
 BseErrorType bse_wave_load_wave (SfiProxy bse_wave_object, const gchar* file_name, const gchar* wave_name);
@@ -755,8 +808,9 @@ SfiProxy bse_wave_use_editable (SfiProxy bse_wave_object, SfiInt chunk_index);
 SfiInt bse_midi_notifier_current_time (SfiProxy bse_midi_notifier_object);
 SfiProxy bse_track_get_part (SfiProxy bse_track_object, SfiInt tick);
 BseSongTiming* bse_track_get_timing (SfiProxy bse_track_object, SfiInt tick);
-BseErrorType bse_track_insert_part (SfiProxy bse_track_object, SfiInt tick, SfiProxy part);
+SfiInt bse_track_insert_part (SfiProxy bse_track_object, SfiInt tick, SfiProxy part);
 BseTrackPartSeq* bse_track_list_parts (SfiProxy bse_track_object);
+void bse_track_remove_link (SfiProxy bse_track_object, SfiInt id);
 void bse_track_remove_tick (SfiProxy bse_track_object, SfiInt tick);
 BseErrorType bse_part_change_note (SfiProxy bse_part_object, SfiInt id, SfiInt tick, SfiInt duration, SfiInt note, SfiInt fine_tune, SfiReal velocity);
 BsePartNoteSeq* bse_part_check_overlap (SfiProxy bse_part_object, SfiInt tick, SfiInt duration, SfiInt note);
@@ -813,12 +867,15 @@ void bse_hello_world (void);
 const gchar* bse_randomizer (SfiBool use_seed, SfiInt seed);
 BseCategory* bse_category_from_id (SfiInt category_id);
 BseCategorySeq* bse_categories_match (const gchar* pattern);
+BseCategorySeq* bse_categories_match_method (const gchar* pattern, const gchar* type);
 BseCategorySeq* bse_categories_match_typed (const gchar* pattern, const gchar* type);
 const gchar* bse_error_blurb (BseErrorType error);
 const gchar* bse_error_name (BseErrorType error);
 SfiBool bse_proxy_check (SfiProxy item, const gchar* type_name);
 SfiInt bse_note_from_freq (SfiReal frequency);
 SfiReal bse_note_to_freq (SfiInt note, SfiInt fine_tune);
+BseSampleFileInfo* bse_sample_file_info (const gchar* file_name);
+SfiReal bse_string_extract_number (const gchar* string, const gchar* format, SfiReal aux_base, SfiReal dflt);
 const gchar* bse_type_blurb (const gchar* type);
 
 
