@@ -38,11 +38,11 @@ enum
 /* --- prototypes --- */
 static void	bse_simple_adsr_init		(BseSimpleADSR		*senv);
 static void	bse_simple_adsr_class_init	(BseSimpleADSRClass	*class);
-static void	bse_simple_adsr_set_property	(BseSimpleADSR		*senv,
+static void	bse_simple_adsr_set_property	(GObject		*object,
 						 guint			 param_id,
-						 GValue			*value,
+						 const GValue		*value,
 						 GParamSpec		*pspec);
-static void	bse_simple_adsr_get_property	(BseSimpleADSR		*simple_adsr,
+static void	bse_simple_adsr_get_property	(GObject                *object,
 						 guint			 param_id,
 						 GValue			*value,
 						 GParamSpec		*pspec);
@@ -83,51 +83,43 @@ bse_simple_adsr_class_init (BseSimpleADSRClass *class)
   
   parent_class = g_type_class_peek_parent (class);
   
-  gobject_class->set_property = (GObjectSetPropertyFunc) bse_simple_adsr_set_property;
-  gobject_class->get_property = (GObjectGetPropertyFunc) bse_simple_adsr_get_property;
+  gobject_class->set_property = bse_simple_adsr_set_property;
+  gobject_class->get_property = bse_simple_adsr_get_property;
   
   source_class->context_create = bse_simple_adsr_context_create;
   
   bse_object_class_add_param (object_class, "Envelope",
 			      PARAM_ATTACK_TIME,
-			      bse_param_spec_float ("attack_time", "Attack Time [%]", NULL,
-						    0.0, 100.0,
-						    10.0, 1.0,
-						    BSE_PARAM_DEFAULT |
-						    BSE_PARAM_HINT_DIAL));
+			      sfi_pspec_real ("attack_time", "Attack Time [%]", NULL,
+					      10.0, 0.0, 100.0, 1.0,
+					      SFI_PARAM_DEFAULT SFI_PARAM_HINT_DIAL SFI_PARAM_FLOAT));
   bse_object_class_add_param (object_class, "Envelope",
 			      PARAM_DECAY_TIME,
-			      bse_param_spec_float ("decay_time", "Decay Time [%]", NULL,
-						    0.0, 100.0,
-						    30.0, 1.0,
-						    BSE_PARAM_DEFAULT |
-						    BSE_PARAM_HINT_DIAL));
+			      sfi_pspec_real ("decay_time", "Decay Time [%]", NULL,
+					      30.0, 0.0, 100.0, 1.0,
+					      SFI_PARAM_DEFAULT SFI_PARAM_HINT_DIAL SFI_PARAM_FLOAT));
   bse_object_class_add_param (object_class, "Envelope",
 			      PARAM_SUSTAIN_LEVEL,
-			      bse_param_spec_float ("sustain_level", "Sustain Level [%]", NULL,
-						    0.0, 100.0,
-						    50.0, 1.0,
-						    BSE_PARAM_DEFAULT |
-						    BSE_PARAM_HINT_DIAL));
+			      sfi_pspec_real ("sustain_level", "Sustain Level [%]", NULL,
+					      50.0, 0.0, 100.0, 1.0,
+					      SFI_PARAM_DEFAULT SFI_PARAM_HINT_DIAL SFI_PARAM_FLOAT));
   bse_object_class_add_param (object_class, "Envelope",
 			      PARAM_RELEASE_TIME,
-			      bse_param_spec_float ("release_time", "Release Time [%]", NULL,
-						    0.0, 100.0,
-						    40.0, 1.0,
-						    BSE_PARAM_DEFAULT |
-						    BSE_PARAM_HINT_DIAL));
+			      sfi_pspec_real ("release_time", "Release Time [%]", NULL,
+					      40.0, 0.0, 100.0, 1.0,
+					      SFI_PARAM_DEFAULT SFI_PARAM_HINT_DIAL SFI_PARAM_FLOAT));
   desc = g_strdup_printf ("Time ranges in seconds: %.1f %.1f %.1f",
 			  BSE_TIME_RANGE_SHORT_ms / 1000.0,
 			  BSE_TIME_RANGE_MEDIUM_ms / 1000.0,
 			  BSE_TIME_RANGE_LONG_ms / 1000.0);
   bse_object_class_add_param (object_class, "Envelope",
 			      PARAM_TIME_RANGE,
-			      bse_param_spec_enum ("time_range", "Time Range", desc,
-						   BSE_TYPE_TIME_RANGE_TYPE,
-						   BSE_TIME_RANGE_SHORT,
-						   BSE_PARAM_DEFAULT));
+			      bse_param_spec_genum ("time_range", "Time Range", desc,
+						    BSE_TYPE_TIME_RANGE_TYPE,
+						    BSE_TIME_RANGE_SHORT,
+						    SFI_PARAM_DEFAULT));
   g_free (desc);
-
+  
   ichannel = bse_source_class_add_ichannel (source_class, "Gate In", "Gate input (activates/deactivates envelope)");
   g_assert (ichannel == BSE_SIMPLE_ADSR_ICHANNEL_GATE);
   ichannel = bse_source_class_add_ichannel (source_class, "Retrigger In", "Retrigger input (raising edge retriggers envelope)");
@@ -149,64 +141,66 @@ bse_simple_adsr_init (BseSimpleADSR *adsr)
 }
 
 static void
-bse_simple_adsr_set_property (BseSimpleADSR *adsr,
-			      guint	     param_id,
-			      GValue	    *value,
-			      GParamSpec    *pspec)
+bse_simple_adsr_set_property (GObject      *object,
+			      guint	    param_id,
+			      const GValue *value,
+			      GParamSpec   *pspec)
 {
+  BseSimpleADSR *self = BSE_SIMPLE_ADSR (object);
   switch (param_id)
     {
     case PARAM_ATTACK_TIME:
-      adsr->attack_time = g_value_get_float (value) / 100.0;
-      bse_simple_adsr_update_modules (adsr, NULL);
+      self->attack_time = sfi_value_get_real (value) / 100.0;
+      bse_simple_adsr_update_modules (self, NULL);
       break;
     case PARAM_DECAY_TIME:
-      adsr->decay_time = g_value_get_float (value) / 100.0;
-      bse_simple_adsr_update_modules (adsr, NULL);
+      self->decay_time = sfi_value_get_real (value) / 100.0;
+      bse_simple_adsr_update_modules (self, NULL);
       break;
     case PARAM_SUSTAIN_LEVEL:
-      adsr->sustain_level = g_value_get_float (value) / 100.0;
-      bse_simple_adsr_update_modules (adsr, NULL);
+      self->sustain_level = sfi_value_get_real (value) / 100.0;
+      bse_simple_adsr_update_modules (self, NULL);
       break;
     case PARAM_RELEASE_TIME:
-      adsr->release_time = g_value_get_float (value) / 100.0;
-      bse_simple_adsr_update_modules (adsr, NULL);
+      self->release_time = sfi_value_get_real (value) / 100.0;
+      bse_simple_adsr_update_modules (self, NULL);
       break;
     case PARAM_TIME_RANGE:
-      adsr->time_range = g_value_get_enum (value);
-      bse_simple_adsr_update_modules (adsr, NULL);
+      self->time_range = g_value_get_enum (value);
+      bse_simple_adsr_update_modules (self, NULL);
       break;
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (adsr, param_id, pspec);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
     }
 }
 
 static void
-bse_simple_adsr_get_property (BseSimpleADSR *adsr,
-			      guint	     param_id,
-			      GValue	    *value,
-			      GParamSpec    *pspec)
+bse_simple_adsr_get_property (GObject    *object,
+			      guint	  param_id,
+			      GValue	 *value,
+			      GParamSpec *pspec)
 {
+  BseSimpleADSR *self = BSE_SIMPLE_ADSR (object);
   switch (param_id)
     {
     case PARAM_ATTACK_TIME:
-      g_value_set_float (value, adsr->attack_time * 100.0);
+      sfi_value_set_real (value, self->attack_time * 100.0);
       break;
     case PARAM_DECAY_TIME:
-      g_value_set_float (value, adsr->decay_time * 100.0);
+      sfi_value_set_real (value, self->decay_time * 100.0);
       break;
     case PARAM_SUSTAIN_LEVEL:
-      g_value_set_float (value, adsr->sustain_level * 100.0);
+      sfi_value_set_real (value, self->sustain_level * 100.0);
       break;
     case PARAM_RELEASE_TIME:
-      g_value_set_float (value, adsr->release_time * 100.0);
+      sfi_value_set_real (value, self->release_time * 100.0);
       break;
     case PARAM_TIME_RANGE:
-      g_value_set_enum (value, adsr->time_range);
+      g_value_set_enum (value, self->time_range);
       break;
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (adsr, param_id, pspec);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
     }
 }
@@ -274,11 +268,11 @@ simple_adsr_process (GslModule *module,
   gfloat *wave_out = GSL_MODULE_OBUFFER (module, BSE_SIMPLE_ADSR_OCHANNEL_OUT);
   gboolean have_gate = GSL_MODULE_ISTREAM (module, BSE_SIMPLE_ADSR_ICHANNEL_GATE).connected;
   guint state = 0;
-
+  
   if (!GSL_MODULE_OSTREAM (module, BSE_SIMPLE_ADSR_OCHANNEL_OUT).connected &&
       !GSL_MODULE_OSTREAM (module, BSE_SIMPLE_ADSR_OCHANNEL_DONE).connected)
     return;	/* no output */
-
+  
   if (env->phase == POST_RELEASE &&
       !GSL_MODULE_ISTREAM (module, BSE_SIMPLE_ADSR_ICHANNEL_GATE).connected &&
       !GSL_MODULE_ISTREAM (module, BSE_SIMPLE_ADSR_ICHANNEL_RETRIGGER).connected)
@@ -291,7 +285,7 @@ simple_adsr_process (GslModule *module,
 	GSL_MODULE_OSTREAM (module, BSE_SIMPLE_ADSR_OCHANNEL_DONE).values = gsl_engine_const_values (0.0);
       return;
     }
-
+  
   ramp->wave_out = wave_out;
   ramp->bound = wave_out + n_values;
   do
@@ -373,7 +367,7 @@ simple_adsr_process (GslModule *module,
 	}
     }
   while (state != BSE_MIX_RAMP_REACHED_BOUND);
-
+  
   if (env->phase == POST_RELEASE)
     GSL_MODULE_OSTREAM (module, BSE_SIMPLE_ADSR_OCHANNEL_DONE).values = gsl_engine_const_values (1.0);
   else
@@ -386,7 +380,7 @@ bse_simple_adsr_update_modules (BseSimpleADSR *adsr,
 {
   BseSimpleADSRVars vars;
   gfloat ms = bse_time_range_to_ms (adsr->time_range);
-
+  
   ms *= gsl_engine_sample_freq () / 1000.0;
   if (adsr->attack_time < TIME_EPSILON)
     {
@@ -447,7 +441,7 @@ bse_simple_adsr_context_create (BseSource *source,
   
   /* chain parent class' handler */
   BSE_SOURCE_CLASS (parent_class)->context_create (source, context_handle, trans);
-
+  
   /* update module data */
   bse_simple_adsr_update_modules (simple_adsr, trans);
 }
